# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: feature_catalog.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import service as _service
from google.protobuf import service_reflection
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from mlflow.protos.scalapb import scalapb_pb2 as scalapb_dot_scalapb__pb2
from mlflow.protos import databricks_pb2 as databricks__pb2
from databricks.feature_store.protos import feature_store_serving_pb2 as feature__store__serving__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x15\x66\x65\x61ture_catalog.proto\x12\x0c\x66\x65\x61turestore\x1a\x15scalapb/scalapb.proto\x1a\x10\x64\x61tabricks.proto\x1a\x1b\x66\x65\x61ture_store_serving.proto\"\x96\x01\n\x0bHealthCheck\x12\x12\n\x03key\x18\x01 \x01(\t:\x05store\x12\x17\n\x05value\x18\x02 \x01(\t:\x08\x66\x65\x61tures\x1a-\n\x08Response\x12\x0e\n\x06result\x18\x01 \x01(\t\x12\x11\n\ttimestamp\x18\x02 \x01(\x03:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\xee\x02\n\x12\x43reateFeatureTable\x12\x12\n\x04name\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12+\n\x0cprimary_keys\x18\x02 \x03(\x0b\x32\x15.featurestore.KeySpec\x12-\n\x0epartition_keys\x18\x03 \x03(\x0b\x32\x15.featurestore.KeySpec\x12\x13\n\x0b\x64\x65scription\x18\x04 \x01(\t\x12\x12\n\ntable_path\x18\x05 \x01(\t\x12-\n\x0etimestamp_keys\x18\x06 \x03(\x0b\x32\x15.featurestore.KeySpec\x12\x13\n\x0bis_imported\x18\x07 \x01(\x08\x12\x16\n\x07\x64ry_run\x18\x08 \x01(\x08:\x05\x66\x61lse\x1a=\n\x08Response\x12\x31\n\rfeature_table\x18\x01 \x01(\x0b\x32\x1a.featurestore.FeatureTable:$\xe2?!\n\x1f\x46\x65\x61tureStoreRPC[$this.Response]\"\xa2\x01\n\x12UpdateFeatureTable\x12\x12\n\x04name\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x13\n\x0b\x64\x65scription\x18\x02 \x01(\t\x1a=\n\x08Response\x12\x31\n\rfeature_table\x18\x01 \x01(\x0b\x32\x1a.featurestore.FeatureTable:$\xe2?!\n\x1f\x46\x65\x61tureStoreRPC[$this.Response]\"r\n\x12\x44\x65leteFeatureTable\x12\x12\n\x04name\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x16\n\x07\x64ry_run\x18\x02 \x01(\x08:\x05\x66\x61lse\x1a\n\n\x08Response:$\xe2?!\n\x1f\x46\x65\x61tureStoreRPC[$this.Response]\"\x8a\x01\n\x0fGetFeatureTable\x12\x12\n\x04name\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x1a=\n\x08Response\x12\x31\n\rfeature_table\x18\x01 \x01(\x0b\x32\x1a.featurestore.FeatureTable:$\xe2?!\n\x1f\x46\x65\x61tureStoreRPC[$this.Response]\"\x97\x02\n\x13SearchFeatureTables\x12\x0c\n\x04text\x18\x01 \x01(\t\x12\x17\n\x0bmax_results\x18\x02 \x01(\x03:\x02\x31\x30\x12\x12\n\npage_token\x18\x03 \x01(\t\x12\x30\n\rsearch_scopes\x18\x04 \x03(\x0e\x32\x19.featurestore.SearchScope\x12\x14\n\x0c\x63\x61talog_name\x18\x05 \x01(\t\x1aW\n\x08Response\x12\x32\n\x0e\x66\x65\x61ture_tables\x18\x01 \x03(\x0b\x32\x1a.featurestore.FeatureTable\x12\x17\n\x0fnext_page_token\x18\x02 \x01(\t:$\xe2?!\n\x1f\x46\x65\x61tureStoreRPC[$this.Response]\"\x89\x01\n\x14GetFeatureTablesById\x12\x0b\n\x03ids\x18\x01 \x03(\t\x1a>\n\x08Response\x12\x32\n\x0e\x66\x65\x61ture_tables\x18\x01 \x03(\x0b\x32\x1a.featurestore.FeatureTable:$\xe2?!\n\x1f\x46\x65\x61tureStoreRPC[$this.Response]\"\xda\x01\n\x0e\x41\x64\x64\x44\x61taSources\x12\x1b\n\rfeature_table\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x0e\n\x06tables\x18\x02 \x03(\t\x12\r\n\x05paths\x18\x03 \x03(\t\x12\x16\n\x0e\x63ustom_sources\x18\x04 \x03(\t\x12\x42\n\x10\x61ssociation_type\x18\x05 \x01(\x0e\x32\x1d.featurestore.AssociationType:\tAUTOMATIC\x1a\n\n\x08Response:$\xe2?!\n\x1f\x46\x65\x61tureStoreRPC[$this.Response]\"s\n\x11\x44\x65leteDataSources\x12\x1b\n\rfeature_table\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x0f\n\x07sources\x18\x02 \x03(\t\x1a\n\n\x08Response:$\xe2?!\n\x1f\x46\x65\x61tureStoreRPC[$this.Response]\"\xaf\x05\n\x13PublishFeatureTable\x12\x1b\n\rfeature_table\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x1a\n\x0conline_table\x18\x02 \x01(\tB\x04\xf8\x86\x19\x01\x12.\n\x05\x63loud\x18\x03 \x01(\x0e\x32\x19.featurestorecommon.CloudB\x04\xf8\x86\x19\x01\x12\x37\n\nstore_type\x18\x04 \x01(\x0e\x32\x1d.featurestorecommon.StoreTypeB\x04\xf8\x86\x19\x01\x12\x0c\n\x04host\x18\x05 \x01(\t\x12\x0c\n\x04port\x18\x06 \x01(\x05\x12\x1a\n\x12read_secret_prefix\x18\x07 \x01(\t\x12\x1b\n\x13write_secret_prefix\x18\x08 \x01(\t\x12;\n\x0emysql_metadata\x18\t \x01(\x0b\x32!.featurestorecommon.MySqlMetadataH\x00\x12\x44\n\x13sql_server_metadata\x18\n \x01(\x0b\x32%.featurestorecommon.SqlServerMetadataH\x00\x12\x41\n\x11\x64ynamodb_metadata\x18\x0b \x01(\x0b\x32$.featurestorecommon.DynamoDbMetadataH\x00\x12\x41\n\x11\x63osmosdb_metadata\x18\r \x01(\x0b\x32$.featurestorecommon.CosmosDbMetadataH\x00\x12\x10\n\x08\x66\x65\x61tures\x18\x0c \x03(\t\x1aI\n\x08Response\x12=\n\x0conline_store\x18\x01 \x01(\x0b\x32\'.featurestorecommon.OnlineStoreDetailed:$\xe2?!\n\x1f\x46\x65\x61tureStoreRPC[$this.Response]B\x15\n\x13\x61\x64\x64itional_metadata\"\xcc\x01\n\x0bGetFeatures\x12\x1b\n\rfeature_table\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x18\n\x0bmax_results\x18\x02 \x01(\x03:\x03\x35\x30\x30\x12\x12\n\npage_token\x18\x03 \x01(\t\x1aL\n\x08Response\x12\'\n\x08\x66\x65\x61tures\x18\x01 \x03(\x0b\x32\x15.featurestore.Feature\x12\x17\n\x0fnext_page_token\x18\x02 \x01(\t:$\xe2?!\n\x1f\x46\x65\x61tureStoreRPC[$this.Response]\"\x97\x01\n\nGetFeature\x12\x1b\n\rfeature_table\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x12\n\x04name\x18\x02 \x01(\tB\x04\xf8\x86\x19\x01\x1a\x32\n\x08Response\x12&\n\x07\x66\x65\x61ture\x18\x01 \x01(\x0b\x32\x15.featurestore.Feature:$\xe2?!\n\x1f\x46\x65\x61tureStoreRPC[$this.Response]\"\x88\x01\n\x0e\x43reateFeatures\x12\x1b\n\rfeature_table\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\'\n\x08\x66\x65\x61tures\x18\x02 \x03(\x0b\x32\x15.featurestore.KeySpec\x1a\n\n\x08Response:$\xe2?!\n\x1f\x46\x65\x61tureStoreRPC[$this.Response]\"\xdd\x01\n\rUpdateFeature\x12\x1b\n\rfeature_table\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x12\n\x04name\x18\x02 \x01(\tB\x04\xf8\x86\x19\x01\x12\x11\n\tdata_type\x18\x03 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x04 \x01(\t\x12\x19\n\x11\x64\x61ta_type_details\x18\x05 \x01(\t\x1a\x32\n\x08Response\x12&\n\x07\x66\x65\x61ture\x18\x01 \x01(\x0b\x32\x15.featurestore.Feature:$\xe2?!\n\x1f\x46\x65\x61tureStoreRPC[$this.Response]\"r\n\rDeleteFeature\x12\x1b\n\rfeature_table\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x12\n\x04name\x18\x02 \x01(\tB\x04\xf8\x86\x19\x01\x1a\n\n\x08Response:$\xe2?!\n\x1f\x46\x65\x61tureStoreRPC[$this.Response]\"\xa4\x02\n\x0b\x41\x64\x64Producer\x12\x1b\n\rfeature_table\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12*\n\x08notebook\x18\x02 \x01(\x0b\x32\x16.featurestore.NotebookH\x00\x12$\n\x07job_run\x18\x03 \x01(\x0b\x32\x11.featurestore.JobH\x00\x12\x31\n\x0c\x64lt_pipeline\x18\x05 \x01(\x0b\x32\x19.featurestore.DltPipelineH\x00\x12\x35\n\x0fproducer_action\x18\x04 \x01(\x0e\x32\x1c.featurestore.ProducerAction\x1a\n\n\x08Response:$\xe2?!\n\x1f\x46\x65\x61tureStoreRPC[$this.Response]B\n\n\x08producer\"\x82\x02\n\x0b\x41\x64\x64\x43onsumer\x12\x30\n\x08\x66\x65\x61tures\x18\x01 \x03(\x0b\x32\x1e.featurestore.ConsumedFeatures\x12*\n\x08notebook\x18\x02 \x01(\x0b\x32\x16.featurestore.NotebookH\x00\x12$\n\x07job_run\x18\x03 \x01(\x0b\x32\x11.featurestore.JobH\x00\x12\x31\n\x0c\x64lt_pipeline\x18\x04 \x01(\x0b\x32\x19.featurestore.DltPipelineH\x00\x1a\n\n\x08Response:$\xe2?!\n\x1f\x46\x65\x61tureStoreRPC[$this.Response]B\n\n\x08\x63onsumer\"\x88\x01\n\x0cGetConsumers\x12\x1b\n\rfeature_table\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x1a\x35\n\x08Response\x12)\n\tconsumers\x18\x01 \x03(\x0b\x32\x16.featurestore.Consumer:$\xe2?!\n\x1f\x46\x65\x61tureStoreRPC[$this.Response]\"\x8b\x01\n\x1eGetFeatureStoreWidePermissions\x1a\x43\n\x08Response\x12\x37\n\x10permission_level\x18\x01 \x01(\x0e\x32\x1d.featurestore.PermissionLevel:$\xe2?!\n\x1f\x46\x65\x61tureStoreRPC[$this.Response]\"\xef\x02\n\x0eGetOnlineStore\x12\x1b\n\rfeature_table\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x1a\n\x0conline_table\x18\x02 \x01(\tB\x04\xf8\x86\x19\x01\x12.\n\x05\x63loud\x18\x03 \x01(\x0e\x32\x19.featurestorecommon.CloudB\x04\xf8\x86\x19\x01\x12\x37\n\nstore_type\x18\x04 \x01(\x0e\x32\x1d.featurestorecommon.StoreTypeB\x04\xf8\x86\x19\x01\x12\x13\n\ttable_arn\x18\x05 \x01(\tH\x00\x12\x17\n\rcontainer_uri\x18\x06 \x01(\tH\x00\x1aI\n\x08Response\x12=\n\x0conline_store\x18\x01 \x01(\x0b\x32\'.featurestorecommon.OnlineStoreDetailed:$\xe2?!\n\x1f\x46\x65\x61tureStoreRPC[$this.Response]B\x1c\n\x1a\x63loud_provider_resource_id\"\xd6\x01\n\x17GetModelServingMetadata\x12\x42\n\x16\x66\x65\x61ture_table_features\x18\x01 \x03(\x0b\x32\".featurestore.FeatureTableFeatures\x1aQ\n\x08Response\x12\x45\n\x15online_feature_tables\x18\x01 \x03(\x0b\x32&.featurestorecommon.OnlineFeatureTable:$\xe2?!\n\x1f\x46\x65\x61tureStoreRPC[$this.Response]\"\x9b\x01\n\x07SetTags\x12\x1a\n\x10\x66\x65\x61ture_table_id\x18\x01 \x01(\tH\x00\x12\x14\n\nfeature_id\x18\x02 \x01(\tH\x00\x12\x1f\n\x04tags\x18\x0b \x03(\x0b\x32\x11.featurestore.Tag\x1a\n\n\x08Response:$\xe2?!\n\x1f\x46\x65\x61tureStoreRPC[$this.Response]B\x0b\n\tentity_id\"\x9b\x01\n\x07GetTags\x12\x1a\n\x10\x66\x65\x61ture_table_id\x18\x01 \x01(\tH\x00\x12\x14\n\nfeature_id\x18\x02 \x01(\tH\x00\x1a+\n\x08Response\x12\x1f\n\x04tags\x18\x01 \x03(\x0b\x32\x11.featurestore.Tag:$\xe2?!\n\x1f\x46\x65\x61tureStoreRPC[$this.Response]B\x0b\n\tentity_id\"\x8b\x01\n\nDeleteTags\x12\x1a\n\x10\x66\x65\x61ture_table_id\x18\x01 \x01(\tH\x00\x12\x14\n\nfeature_id\x18\x02 \x01(\tH\x00\x12\x0c\n\x04keys\x18\x0b \x03(\t\x1a\n\n\x08Response:$\xe2?!\n\x1f\x46\x65\x61tureStoreRPC[$this.Response]B\x0b\n\tentity_id\"Q\n\x07KeySpec\x12\x12\n\x04name\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x17\n\tdata_type\x18\x02 \x01(\tB\x04\xf8\x86\x19\x01\x12\x19\n\x11\x64\x61ta_type_details\x18\x03 \x01(\t\"\xd1\x04\n\x0c\x46\x65\x61tureTable\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x1a\n\x12\x63reation_timestamp\x18\x02 \x01(\x03\x12\x1e\n\x16last_updated_timestamp\x18\x03 \x01(\x03\x12\x12\n\ncreator_id\x18\x04 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x05 \x01(\t\x12\x14\n\x0cprimary_keys\x18\x06 \x03(\t\x12\x16\n\x0epartition_keys\x18\x07 \x03(\t\x12\x10\n\x08\x66\x65\x61tures\x18\x08 \x03(\t\x12.\n\x0c\x64\x61ta_sources\x18\t \x03(\x0b\x32\x18.featurestore.DataSource\x12\x36\n\ronline_stores\x18\n \x03(\x0b\x32\x1f.featurestorecommon.OnlineStore\x12\x32\n\x12notebook_producers\x18\x0b \x03(\x0b\x32\x16.featurestore.Notebook\x12(\n\rjob_producers\x18\x0c \x03(\x0b\x32\x11.featurestore.Job\x12\x39\n\x16\x64lt_pipeline_producers\x18\x12 \x03(\x0b\x32\x19.featurestore.DltPipeline\x12\x1b\n\x13last_update_user_id\x18\r \x01(\t\x12\n\n\x02id\x18\x0e \x01(\t\x12\x37\n\x10permission_level\x18\x0f \x01(\x0e\x32\x1d.featurestore.PermissionLevel\x12\x16\n\x0etimestamp_keys\x18\x10 \x03(\t\x12\x13\n\x0bis_imported\x18\x11 \x01(\x08\"\x80\x02\n\x07\x46\x65\x61ture\x12\r\n\x05table\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\x12/\n\tdata_type\x18\x03 \x01(\x0e\x32\x1c.featurestorecommon.DataType\x12\x13\n\x0b\x64\x65scription\x18\x04 \x01(\t\x12\x19\n\x11\x64\x61ta_type_details\x18\x05 \x01(\t\x12\n\n\x02id\x18\x06 \x01(\t\x12\x1a\n\x12\x63reation_timestamp\x18\x07 \x01(\x03\x12\x12\n\ncreator_id\x18\x08 \x01(\t\x12\x1e\n\x16last_updated_timestamp\x18\t \x01(\x03\x12\x1b\n\x13last_update_user_id\x18\n \x01(\t\"@\n\nDataSource\x12\r\n\x05table\x18\x01 \x01(\t\x12\x0c\n\x04path\x18\x02 \x01(\t\x12\x15\n\rcustom_source\x18\x03 \x01(\t\"0\n\x10\x43onsumedFeatures\x12\r\n\x05table\x18\x01 \x01(\t\x12\r\n\x05names\x18\x02 \x03(\t\"\x9a\x02\n\x08Notebook\x12\x13\n\x0bnotebook_id\x18\x01 \x01(\x03\x12\x13\n\x0brevision_id\x18\x02 \x01(\x03\x12\x1a\n\x0eworkspace_path\x18\x03 \x01(\tB\x02\x18\x01\x12\x1a\n\x12\x63reation_timestamp\x18\x04 \x01(\x03\x12\x12\n\ncreator_id\x18\x05 \x01(\t\x12\x1d\n\x15notebook_workspace_id\x18\x06 \x01(\x03\x12\"\n\x1a\x66\x65\x61ture_table_workspace_id\x18\x07 \x01(\x03\x12\x1e\n\x16notebook_workspace_url\x18\x08 \x01(\t\x12\x35\n\x0fproducer_action\x18\t \x01(\x0e\x32\x1c.featurestore.ProducerAction\"\xfb\x01\n\x03Job\x12\x0e\n\x06job_id\x18\x01 \x01(\x03\x12\x0e\n\x06run_id\x18\x02 \x01(\x03\x12\x14\n\x08job_name\x18\x03 \x01(\tB\x02\x18\x01\x12\x1a\n\x12\x63reation_timestamp\x18\x04 \x01(\x03\x12\x12\n\ncreator_id\x18\x05 \x01(\t\x12\x18\n\x10job_workspace_id\x18\x06 \x01(\x03\x12\"\n\x1a\x66\x65\x61ture_table_workspace_id\x18\x07 \x01(\x03\x12\x19\n\x11job_workspace_url\x18\x08 \x01(\t\x12\x35\n\x0fproducer_action\x18\t \x01(\x0e\x32\x1c.featurestore.ProducerAction\"\x9c\x01\n\x0b\x44ltPipeline\x12\x13\n\x0bpipeline_id\x18\x01 \x01(\t\x12\x11\n\tupdate_id\x18\x02 \x01(\t\x12\x1a\n\x12\x63reation_timestamp\x18\x03 \x01(\x03\x12\x12\n\ncreator_id\x18\x04 \x01(\t\x12\x35\n\x0fproducer_action\x18\x05 \x01(\x0e\x32\x1c.featurestore.ProducerAction\"\xab\x01\n\x08\x43onsumer\x12\x10\n\x08\x66\x65\x61tures\x18\x01 \x03(\t\x12*\n\x08notebook\x18\x02 \x01(\x0b\x32\x16.featurestore.NotebookH\x00\x12$\n\x07job_run\x18\x03 \x01(\x0b\x32\x11.featurestore.JobH\x00\x12\x31\n\x0c\x64lt_pipeline\x18\x04 \x01(\x0b\x32\x19.featurestore.DltPipelineH\x00\x42\x08\n\x06\x65ntity\"D\n\x14\x46\x65\x61tureTableFeatures\x12\x1a\n\x12\x66\x65\x61ture_table_name\x18\x01 \x01(\t\x12\x10\n\x08\x66\x65\x61tures\x18\x02 \x03(\t\"!\n\x03Tag\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t\"\x90\x01\n\x10MonitoringConfig\x12\x1b\n\rfeature_table\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x10\n\x08\x66\x65\x61tures\x18\x02 \x03(\t\x12/\n\x0crefresh_mode\x18\x03 \x01(\x0e\x32\x19.featurestore.RefreshMode\x12\x1c\n\x14window_granularities\x18\x04 \x03(\t\"\xd4\x01\n\x07Profile\x12\x12\n\nprofile_id\x18\x01 \x01(\t\x12\x15\n\rfeature_table\x18\x02 \x01(\t\x12\x1e\n\x16window_start_timestamp\x18\x03 \x01(\x03\x12\x1a\n\x12window_granularity\x18\x04 \x01(\t\x12-\n\x0btable_stats\x18\x05 \x01(\x0b\x32\x18.featurestore.TableStats\x12\x17\n\x0f\x63olumn_profiles\x18\x06 \x03(\t\x12\x1a\n\x12\x63reation_timestamp\x18\x07 \x01(\x03\"\x1e\n\nTableStats\x12\x10\n\x08num_rows\x18\x01 \x01(\x03\"\x80\x02\n\x13SetMonitoringConfig\x12\x1b\n\rfeature_table\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x10\n\x08\x66\x65\x61tures\x18\x02 \x03(\t\x12/\n\x0crefresh_mode\x18\x03 \x01(\x0e\x32\x19.featurestore.RefreshMode\x12\x1c\n\x14window_granularities\x18\x04 \x03(\t\x1a\x45\n\x08Response\x12\x39\n\x11monitoring_config\x18\x01 \x01(\x0b\x32\x1e.featurestore.MonitoringConfig:$\xe2?!\n\x1f\x46\x65\x61tureStoreRPC[$this.Response]\"\x9f\x01\n\x13GetMonitoringConfig\x12\x1b\n\rfeature_table\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x1a\x45\n\x08Response\x12\x39\n\x11monitoring_config\x18\x01 \x01(\x0b\x32\x1e.featurestore.MonitoringConfig:$\xe2?!\n\x1f\x46\x65\x61tureStoreRPC[$this.Response]\"\xcc\x01\n\x0cListProfiles\x12\x1b\n\rfeature_table\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x17\n\x0bmax_results\x18\x02 \x01(\x03:\x02\x35\x30\x12\x12\n\npage_token\x18\x03 \x01(\t\x1aL\n\x08Response\x12\'\n\x08profiles\x18\x01 \x03(\x0b\x32\x15.featurestore.Profile\x12\x17\n\x0fnext_page_token\x18\x02 \x01(\t:$\xe2?!\n\x1f\x46\x65\x61tureStoreRPC[$this.Response]\"\x97\x01\n\nGetProfile\x12\x1b\n\rfeature_table\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x12\n\nprofile_id\x18\x02 \x01(\t\x1a\x32\n\x08Response\x12&\n\x07profile\x18\x01 \x01(\x0b\x32\x15.featurestore.Profile:$\xe2?!\n\x1f\x46\x65\x61tureStoreRPC[$this.Response]\"\x84\x02\n\rCreateProfile\x12\x15\n\rfeature_table\x18\x01 \x01(\t\x12\x1e\n\x16window_start_timestamp\x18\x02 \x01(\x03\x12\x1a\n\x12window_granularity\x18\x03 \x01(\t\x12-\n\x0btable_stats\x18\x04 \x01(\x0b\x32\x18.featurestore.TableStats\x12\x17\n\x0f\x63olumn_profiles\x18\x05 \x03(\t\x1a\x32\n\x08Response\x12&\n\x07profile\x18\x01 \x01(\x0b\x32\x15.featurestore.Profile:$\xe2?!\n\x1f\x46\x65\x61tureStoreRPC[$this.Response]\"v\n\x0e\x44\x65leteProfiles\x12\x15\n\rfeature_table\x18\x01 \x01(\t\x1a\'\n\x08Response\x12\x1b\n\x13\x64\x65leted_profile_ids\x18\x01 \x03(\t:$\xe2?!\n\x1f\x46\x65\x61tureStoreRPC[$this.Response]*_\n\x0fPermissionLevel\x12\x0e\n\nCAN_MANAGE\x10\x00\x12\x15\n\x11\x43\x41N_EDIT_METADATA\x10\x01\x12\x15\n\x11\x43\x41N_VIEW_METADATA\x10\x02\x12\x0e\n\nCAN_CREATE\x10\x03*5\n\x0eProducerAction\x12\n\n\x06\x43REATE\x10\x00\x12\t\n\x05WRITE\x10\x01\x12\x0c\n\x08REGISTER\x10\x02*,\n\x0f\x41ssociationType\x12\r\n\tAUTOMATIC\x10\x00\x12\n\n\x06MANUAL\x10\x01*k\n\x0bSearchScope\x12\x12\n\x0e\x46\x45\x41TURE_TABLES\x10\x01\x12\x0c\n\x08\x46\x45\x41TURES\x10\x02\x12\x10\n\x0c\x44\x41TA_SOURCES\x10\x03\x12\x16\n\x12\x46\x45\x41TURE_TABLE_TAGS\x10\x04\x12\x10\n\x0c\x46\x45\x41TURE_TAGS\x10\x05*3\n\x0bRefreshMode\x12\x12\n\x0eMANUAL_REFRESH\x10\x00\x12\x10\n\x0c\x41UTO_REFRESH\x10\x01\x32\xa3*\n\x13\x46\x65\x61tureStoreService\x12\xab\x01\n\x0bhealthCheck\x12\x19.featurestore.HealthCheck\x1a\".featurestore.HealthCheck.Response\"]\xf2\x86\x19Y\n(\n\x03GET\x12\x1b/feature-store/health-check\x1a\x04\x08\x02\x10\x00\x10\x03*+Health check for Feature Store service pod.\x12\xb4\x01\n\x12\x63reateFeatureTable\x12 .featurestore.CreateFeatureTable\x1a).featurestore.CreateFeatureTable.Response\"Q\xf2\x86\x19M\n2\n\x04POST\x12$/feature-store/feature-tables/create\x1a\x04\x08\x02\x10\x00\x10\x03*\x15\x43reate Feature Table.\x12\xb5\x01\n\x12updateFeatureTable\x12 .featurestore.UpdateFeatureTable\x1a).featurestore.UpdateFeatureTable.Response\"R\xf2\x86\x19N\n3\n\x05PATCH\x12$/feature-store/feature-tables/update\x1a\x04\x08\x02\x10\x00\x10\x03*\x15Update Feature Table.\x12\xb6\x01\n\x12\x64\x65leteFeatureTable\x12 .featurestore.DeleteFeatureTable\x1a).featurestore.DeleteFeatureTable.Response\"S\xf2\x86\x19O\n4\n\x06\x44\x45LETE\x12$/feature-store/feature-tables/delete\x1a\x04\x08\x02\x10\x00\x10\x03*\x15\x44\x65lete Feature Table.\x12\xa4\x01\n\x0fgetFeatureTable\x12\x1d.featurestore.GetFeatureTable\x1a&.featurestore.GetFeatureTable.Response\"J\xf2\x86\x19\x46\n.\n\x03GET\x12!/feature-store/feature-tables/get\x1a\x04\x08\x02\x10\x00\x10\x03*\x12Get Feature Table.\x12\xed\x01\n\x13searchFeatureTables\x12!.featurestore.SearchFeatureTables\x1a*.featurestore.SearchFeatureTables.Response\"\x86\x01\xf2\x86\x19\x81\x01\n1\n\x03GET\x12$/feature-store/feature-tables/search\x1a\x04\x08\x02\x10\x00\n2\n\x04POST\x12$/feature-store/feature-tables/search\x1a\x04\x08\x02\x10\x00\x10\x03*\x16Search Feature Tables.\x12\xc2\x01\n\x14getFeatureTablesById\x12\".featurestore.GetFeatureTablesById\x1a+.featurestore.GetFeatureTablesById.Response\"Y\xf2\x86\x19U\n6\n\x04POST\x12(/feature-store/feature-tables/get-by-ids\x1a\x04\x08\x02\x10\x00\x10\x03*\x19Get Feature Tables by ID.\x12\xc0\x01\n\x0e\x61\x64\x64\x44\x61taSources\x12\x1c.featurestore.AddDataSources\x1a%.featurestore.AddDataSources.Response\"i\xf2\x86\x19\x65\n<\n\x04POST\x12./feature-store/feature-tables/add-data-sources\x1a\x04\x08\x02\x10\x00\x10\x03*#Add Data Sources for Feature Table.\x12\xcf\x01\n\x11\x64\x65leteDataSources\x12\x1f.featurestore.DeleteDataSources\x1a(.featurestore.DeleteDataSources.Response\"o\xf2\x86\x19k\n?\n\x04POST\x12\x31/feature-store/feature-tables/delete-data-sources\x1a\x04\x08\x02\x10\x00\x10\x03*&Delete Data Sources for Feature Table.\x12\xb9\x01\n\x13publishFeatureTable\x12!.featurestore.PublishFeatureTable\x1a*.featurestore.PublishFeatureTable.Response\"S\xf2\x86\x19O\n3\n\x04POST\x12%/feature-store/feature-tables/publish\x1a\x04\x08\x02\x10\x00\x10\x03*\x16Publish Feature Table.\x12\x8d\x01\n\x0bgetFeatures\x12\x19.featurestore.GetFeatures\x1a\".featurestore.GetFeatures.Response\"?\xf2\x86\x19;\n(\n\x03GET\x12\x1b/feature-store/features/get\x1a\x04\x08\x02\x10\x00\x10\x03*\rGet Features.\x12\x91\x01\n\ngetFeature\x12\x18.featurestore.GetFeature\x1a!.featurestore.GetFeature.Response\"F\xf2\x86\x19\x42\n0\n\x03GET\x12#/feature-store/features/get-by-name\x1a\x04\x08\x02\x10\x00\x10\x03*\x0cGet Feature.\x12\x9d\x01\n\x0e\x63reateFeatures\x12\x1c.featurestore.CreateFeatures\x1a%.featurestore.CreateFeatures.Response\"F\xf2\x86\x19\x42\n,\n\x04POST\x12\x1e/feature-store/features/create\x1a\x04\x08\x02\x10\x00\x10\x03*\x10\x43reate Features.\x12\x9a\x01\n\rupdateFeature\x12\x1b.featurestore.UpdateFeature\x1a$.featurestore.UpdateFeature.Response\"F\xf2\x86\x19\x42\n-\n\x05PATCH\x12\x1e/feature-store/features/update\x1a\x04\x08\x02\x10\x00\x10\x03*\x0fUpdate Feature.\x12\x9b\x01\n\rdeleteFeature\x12\x1b.featurestore.DeleteFeature\x1a$.featurestore.DeleteFeature.Response\"G\xf2\x86\x19\x43\n.\n\x06\x44\x45LETE\x12\x1e/feature-store/features/delete\x1a\x04\x08\x02\x10\x00\x10\x03*\x0f\x44\x65lete Feature.\x12\x9d\x01\n\x0b\x61\x64\x64Producer\x12\x19.featurestore.AddProducer\x1a\".featurestore.AddProducer.Response\"O\xf2\x86\x19K\n8\n\x04POST\x12*/feature-store/feature-tables/add-producer\x1a\x04\x08\x02\x10\x00\x10\x03*\rAdd Producer.\x12\x97\x01\n\x0b\x61\x64\x64\x43onsumer\x12\x19.featurestore.AddConsumer\x1a\".featurestore.AddConsumer.Response\"I\xf2\x86\x19\x45\n2\n\x04POST\x12$/feature-store/features/add-consumer\x1a\x04\x08\x02\x10\x00\x10\x03*\rAdd Consumer.\x12\xa1\x01\n\x0cgetConsumers\x12\x1a.featurestore.GetConsumers\x1a#.featurestore.GetConsumers.Response\"P\xf2\x86\x19L\n8\n\x03GET\x12+/feature-store/feature-tables/get-consumers\x1a\x04\x08\x02\x10\x00\x10\x03*\x0eGet Consumers.\x12\x9d\x02\n\x1egetFeatureStoreWidePermissions\x12,.featurestore.GetFeatureStoreWidePermissions\x1a\x35.featurestore.GetFeatureStoreWidePermissions.Response\"\x95\x01\xf2\x86\x19\x90\x01\nM\n\x03GET\x12@/feature-store/feature-tables/get-feature-store-wide-permissions\x1a\x04\x08\x02\x10\x00\x10\x03*=Get Feature Store Wide Permission Level for the Current User.\x12\xad\x01\n\x0egetOnlineStore\x12\x1c.featurestore.GetOnlineStore\x1a%.featurestore.GetOnlineStore.Response\"V\xf2\x86\x19R\n;\n\x03GET\x12./feature-store/feature-tables/get-online-store\x1a\x04\x08\x02\x10\x00\x10\x03*\x11Get Online Store.\x12\x88\x02\n\x17getModelServingMetadata\x12%.featurestore.GetModelServingMetadata\x1a..featurestore.GetModelServingMetadata.Response\"\x95\x01\xf2\x86\x19\x90\x01\n7\n\x04POST\x12)/feature-store/model-serving/get-metadata\x1a\x04\x08\x02\x10\x00\n6\n\x03GET\x12)/feature-store/model-serving/get-metadata\x1a\x04\x08\x02\x10\x00\x10\x03*\x1bGet Model Serving Metadata.\x12z\n\x07setTags\x12\x15.featurestore.SetTags\x1a\x1e.featurestore.SetTags.Response\"8\xf2\x86\x19\x34\n%\n\x04POST\x12\x17/feature-store/tags/set\x1a\x04\x08\x02\x10\x00\x10\x03*\tSet Tags.\x12y\n\x07getTags\x12\x15.featurestore.GetTags\x1a\x1e.featurestore.GetTags.Response\"7\xf2\x86\x19\x33\n$\n\x03GET\x12\x17/feature-store/tags/get\x1a\x04\x08\x02\x10\x00\x10\x03*\tGet Tags.\x12\x8b\x01\n\ndeleteTags\x12\x18.featurestore.DeleteTags\x1a!.featurestore.DeleteTags.Response\"@\xf2\x86\x19<\n*\n\x06\x44\x45LETE\x12\x1a/feature-store/tags/delete\x1a\x04\x08\x02\x10\x00\x10\x03*\x0c\x44\x65lete Tags.\x12\xc1\x01\n\x13setMonitoringConfig\x12!.featurestore.SetMonitoringConfig\x1a*.featurestore.SetMonitoringConfig.Response\"[\xf2\x86\x19W\n;\n\x04POST\x12\x33/feature-store/feature-tables/set-monitoring-config\x10\x03*\x16Set Monitoring Config.\x12\xc0\x01\n\x13getMonitoringConfig\x12!.featurestore.GetMonitoringConfig\x1a*.featurestore.GetMonitoringConfig.Response\"Z\xf2\x86\x19V\n:\n\x03GET\x12\x33/feature-store/feature-tables/get-monitoring-config\x10\x03*\x16Get Monitoring Config.\x12\xb4\x01\n\x0clistProfiles\x12\x1a.featurestore.ListProfiles\x1a#.featurestore.ListProfiles.Response\"c\xf2\x86\x19_\n#\n\x03GET\x12\x1c/feature-store/profiles/list\x10\x03*6List all the profiles that belongs to a feature table.\x12\xa2\x01\n\ngetProfile\x12\x18.featurestore.GetProfile\x1a!.featurestore.GetProfile.Response\"W\xf2\x86\x19S\n\"\n\x03GET\x12\x1b/feature-store/profiles/get\x10\x03*+Fetch the data profile for a feature table.\x12\xac\x01\n\rcreateProfile\x12\x1b.featurestore.CreateProfile\x1a$.featurestore.CreateProfile.Response\"X\xf2\x86\x19T\n&\n\x04POST\x12\x1e/feature-store/profiles/create\x10\x03*(Save a data profile for a feature table.\x12\xb9\x01\n\x0e\x64\x65leteProfiles\x12\x1c.featurestore.DeleteProfiles\x1a%.featurestore.DeleteProfiles.Response\"b\xf2\x86\x19^\n+\n\x06\x44\x45LETE\x12!/feature-store/profiles/deleteAll\x10\x03*-Delete all data profiles for a feature table.Bc\n%com.databricks.api.proto.featurestore\x90\x01\x01\xa0\x01\x01\xe2?3\x10\x01\x1a/com.databricks.featurestore.api.FeatureStoreRPC')

_PERMISSIONLEVEL = DESCRIPTOR.enum_types_by_name['PermissionLevel']
PermissionLevel = enum_type_wrapper.EnumTypeWrapper(_PERMISSIONLEVEL)
_PRODUCERACTION = DESCRIPTOR.enum_types_by_name['ProducerAction']
ProducerAction = enum_type_wrapper.EnumTypeWrapper(_PRODUCERACTION)
_ASSOCIATIONTYPE = DESCRIPTOR.enum_types_by_name['AssociationType']
AssociationType = enum_type_wrapper.EnumTypeWrapper(_ASSOCIATIONTYPE)
_SEARCHSCOPE = DESCRIPTOR.enum_types_by_name['SearchScope']
SearchScope = enum_type_wrapper.EnumTypeWrapper(_SEARCHSCOPE)
_REFRESHMODE = DESCRIPTOR.enum_types_by_name['RefreshMode']
RefreshMode = enum_type_wrapper.EnumTypeWrapper(_REFRESHMODE)
CAN_MANAGE = 0
CAN_EDIT_METADATA = 1
CAN_VIEW_METADATA = 2
CAN_CREATE = 3
CREATE = 0
WRITE = 1
REGISTER = 2
AUTOMATIC = 0
MANUAL = 1
FEATURE_TABLES = 1
FEATURES = 2
DATA_SOURCES = 3
FEATURE_TABLE_TAGS = 4
FEATURE_TAGS = 5
MANUAL_REFRESH = 0
AUTO_REFRESH = 1


_HEALTHCHECK = DESCRIPTOR.message_types_by_name['HealthCheck']
_HEALTHCHECK_RESPONSE = _HEALTHCHECK.nested_types_by_name['Response']
_CREATEFEATURETABLE = DESCRIPTOR.message_types_by_name['CreateFeatureTable']
_CREATEFEATURETABLE_RESPONSE = _CREATEFEATURETABLE.nested_types_by_name['Response']
_UPDATEFEATURETABLE = DESCRIPTOR.message_types_by_name['UpdateFeatureTable']
_UPDATEFEATURETABLE_RESPONSE = _UPDATEFEATURETABLE.nested_types_by_name['Response']
_DELETEFEATURETABLE = DESCRIPTOR.message_types_by_name['DeleteFeatureTable']
_DELETEFEATURETABLE_RESPONSE = _DELETEFEATURETABLE.nested_types_by_name['Response']
_GETFEATURETABLE = DESCRIPTOR.message_types_by_name['GetFeatureTable']
_GETFEATURETABLE_RESPONSE = _GETFEATURETABLE.nested_types_by_name['Response']
_SEARCHFEATURETABLES = DESCRIPTOR.message_types_by_name['SearchFeatureTables']
_SEARCHFEATURETABLES_RESPONSE = _SEARCHFEATURETABLES.nested_types_by_name['Response']
_GETFEATURETABLESBYID = DESCRIPTOR.message_types_by_name['GetFeatureTablesById']
_GETFEATURETABLESBYID_RESPONSE = _GETFEATURETABLESBYID.nested_types_by_name['Response']
_ADDDATASOURCES = DESCRIPTOR.message_types_by_name['AddDataSources']
_ADDDATASOURCES_RESPONSE = _ADDDATASOURCES.nested_types_by_name['Response']
_DELETEDATASOURCES = DESCRIPTOR.message_types_by_name['DeleteDataSources']
_DELETEDATASOURCES_RESPONSE = _DELETEDATASOURCES.nested_types_by_name['Response']
_PUBLISHFEATURETABLE = DESCRIPTOR.message_types_by_name['PublishFeatureTable']
_PUBLISHFEATURETABLE_RESPONSE = _PUBLISHFEATURETABLE.nested_types_by_name['Response']
_GETFEATURES = DESCRIPTOR.message_types_by_name['GetFeatures']
_GETFEATURES_RESPONSE = _GETFEATURES.nested_types_by_name['Response']
_GETFEATURE = DESCRIPTOR.message_types_by_name['GetFeature']
_GETFEATURE_RESPONSE = _GETFEATURE.nested_types_by_name['Response']
_CREATEFEATURES = DESCRIPTOR.message_types_by_name['CreateFeatures']
_CREATEFEATURES_RESPONSE = _CREATEFEATURES.nested_types_by_name['Response']
_UPDATEFEATURE = DESCRIPTOR.message_types_by_name['UpdateFeature']
_UPDATEFEATURE_RESPONSE = _UPDATEFEATURE.nested_types_by_name['Response']
_DELETEFEATURE = DESCRIPTOR.message_types_by_name['DeleteFeature']
_DELETEFEATURE_RESPONSE = _DELETEFEATURE.nested_types_by_name['Response']
_ADDPRODUCER = DESCRIPTOR.message_types_by_name['AddProducer']
_ADDPRODUCER_RESPONSE = _ADDPRODUCER.nested_types_by_name['Response']
_ADDCONSUMER = DESCRIPTOR.message_types_by_name['AddConsumer']
_ADDCONSUMER_RESPONSE = _ADDCONSUMER.nested_types_by_name['Response']
_GETCONSUMERS = DESCRIPTOR.message_types_by_name['GetConsumers']
_GETCONSUMERS_RESPONSE = _GETCONSUMERS.nested_types_by_name['Response']
_GETFEATURESTOREWIDEPERMISSIONS = DESCRIPTOR.message_types_by_name['GetFeatureStoreWidePermissions']
_GETFEATURESTOREWIDEPERMISSIONS_RESPONSE = _GETFEATURESTOREWIDEPERMISSIONS.nested_types_by_name['Response']
_GETONLINESTORE = DESCRIPTOR.message_types_by_name['GetOnlineStore']
_GETONLINESTORE_RESPONSE = _GETONLINESTORE.nested_types_by_name['Response']
_GETMODELSERVINGMETADATA = DESCRIPTOR.message_types_by_name['GetModelServingMetadata']
_GETMODELSERVINGMETADATA_RESPONSE = _GETMODELSERVINGMETADATA.nested_types_by_name['Response']
_SETTAGS = DESCRIPTOR.message_types_by_name['SetTags']
_SETTAGS_RESPONSE = _SETTAGS.nested_types_by_name['Response']
_GETTAGS = DESCRIPTOR.message_types_by_name['GetTags']
_GETTAGS_RESPONSE = _GETTAGS.nested_types_by_name['Response']
_DELETETAGS = DESCRIPTOR.message_types_by_name['DeleteTags']
_DELETETAGS_RESPONSE = _DELETETAGS.nested_types_by_name['Response']
_KEYSPEC = DESCRIPTOR.message_types_by_name['KeySpec']
_FEATURETABLE = DESCRIPTOR.message_types_by_name['FeatureTable']
_FEATURE = DESCRIPTOR.message_types_by_name['Feature']
_DATASOURCE = DESCRIPTOR.message_types_by_name['DataSource']
_CONSUMEDFEATURES = DESCRIPTOR.message_types_by_name['ConsumedFeatures']
_NOTEBOOK = DESCRIPTOR.message_types_by_name['Notebook']
_JOB = DESCRIPTOR.message_types_by_name['Job']
_DLTPIPELINE = DESCRIPTOR.message_types_by_name['DltPipeline']
_CONSUMER = DESCRIPTOR.message_types_by_name['Consumer']
_FEATURETABLEFEATURES = DESCRIPTOR.message_types_by_name['FeatureTableFeatures']
_TAG = DESCRIPTOR.message_types_by_name['Tag']
_MONITORINGCONFIG = DESCRIPTOR.message_types_by_name['MonitoringConfig']
_PROFILE = DESCRIPTOR.message_types_by_name['Profile']
_TABLESTATS = DESCRIPTOR.message_types_by_name['TableStats']
_SETMONITORINGCONFIG = DESCRIPTOR.message_types_by_name['SetMonitoringConfig']
_SETMONITORINGCONFIG_RESPONSE = _SETMONITORINGCONFIG.nested_types_by_name['Response']
_GETMONITORINGCONFIG = DESCRIPTOR.message_types_by_name['GetMonitoringConfig']
_GETMONITORINGCONFIG_RESPONSE = _GETMONITORINGCONFIG.nested_types_by_name['Response']
_LISTPROFILES = DESCRIPTOR.message_types_by_name['ListProfiles']
_LISTPROFILES_RESPONSE = _LISTPROFILES.nested_types_by_name['Response']
_GETPROFILE = DESCRIPTOR.message_types_by_name['GetProfile']
_GETPROFILE_RESPONSE = _GETPROFILE.nested_types_by_name['Response']
_CREATEPROFILE = DESCRIPTOR.message_types_by_name['CreateProfile']
_CREATEPROFILE_RESPONSE = _CREATEPROFILE.nested_types_by_name['Response']
_DELETEPROFILES = DESCRIPTOR.message_types_by_name['DeleteProfiles']
_DELETEPROFILES_RESPONSE = _DELETEPROFILES.nested_types_by_name['Response']
HealthCheck = _reflection.GeneratedProtocolMessageType('HealthCheck', (_message.Message,), {

  'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
    'DESCRIPTOR' : _HEALTHCHECK_RESPONSE,
    '__module__' : 'feature_catalog_pb2'
    # @@protoc_insertion_point(class_scope:featurestore.HealthCheck.Response)
    })
  ,
  'DESCRIPTOR' : _HEALTHCHECK,
  '__module__' : 'feature_catalog_pb2'
  # @@protoc_insertion_point(class_scope:featurestore.HealthCheck)
  })
_sym_db.RegisterMessage(HealthCheck)
_sym_db.RegisterMessage(HealthCheck.Response)

CreateFeatureTable = _reflection.GeneratedProtocolMessageType('CreateFeatureTable', (_message.Message,), {

  'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
    'DESCRIPTOR' : _CREATEFEATURETABLE_RESPONSE,
    '__module__' : 'feature_catalog_pb2'
    # @@protoc_insertion_point(class_scope:featurestore.CreateFeatureTable.Response)
    })
  ,
  'DESCRIPTOR' : _CREATEFEATURETABLE,
  '__module__' : 'feature_catalog_pb2'
  # @@protoc_insertion_point(class_scope:featurestore.CreateFeatureTable)
  })
_sym_db.RegisterMessage(CreateFeatureTable)
_sym_db.RegisterMessage(CreateFeatureTable.Response)

UpdateFeatureTable = _reflection.GeneratedProtocolMessageType('UpdateFeatureTable', (_message.Message,), {

  'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
    'DESCRIPTOR' : _UPDATEFEATURETABLE_RESPONSE,
    '__module__' : 'feature_catalog_pb2'
    # @@protoc_insertion_point(class_scope:featurestore.UpdateFeatureTable.Response)
    })
  ,
  'DESCRIPTOR' : _UPDATEFEATURETABLE,
  '__module__' : 'feature_catalog_pb2'
  # @@protoc_insertion_point(class_scope:featurestore.UpdateFeatureTable)
  })
_sym_db.RegisterMessage(UpdateFeatureTable)
_sym_db.RegisterMessage(UpdateFeatureTable.Response)

DeleteFeatureTable = _reflection.GeneratedProtocolMessageType('DeleteFeatureTable', (_message.Message,), {

  'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
    'DESCRIPTOR' : _DELETEFEATURETABLE_RESPONSE,
    '__module__' : 'feature_catalog_pb2'
    # @@protoc_insertion_point(class_scope:featurestore.DeleteFeatureTable.Response)
    })
  ,
  'DESCRIPTOR' : _DELETEFEATURETABLE,
  '__module__' : 'feature_catalog_pb2'
  # @@protoc_insertion_point(class_scope:featurestore.DeleteFeatureTable)
  })
_sym_db.RegisterMessage(DeleteFeatureTable)
_sym_db.RegisterMessage(DeleteFeatureTable.Response)

GetFeatureTable = _reflection.GeneratedProtocolMessageType('GetFeatureTable', (_message.Message,), {

  'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
    'DESCRIPTOR' : _GETFEATURETABLE_RESPONSE,
    '__module__' : 'feature_catalog_pb2'
    # @@protoc_insertion_point(class_scope:featurestore.GetFeatureTable.Response)
    })
  ,
  'DESCRIPTOR' : _GETFEATURETABLE,
  '__module__' : 'feature_catalog_pb2'
  # @@protoc_insertion_point(class_scope:featurestore.GetFeatureTable)
  })
_sym_db.RegisterMessage(GetFeatureTable)
_sym_db.RegisterMessage(GetFeatureTable.Response)

SearchFeatureTables = _reflection.GeneratedProtocolMessageType('SearchFeatureTables', (_message.Message,), {

  'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
    'DESCRIPTOR' : _SEARCHFEATURETABLES_RESPONSE,
    '__module__' : 'feature_catalog_pb2'
    # @@protoc_insertion_point(class_scope:featurestore.SearchFeatureTables.Response)
    })
  ,
  'DESCRIPTOR' : _SEARCHFEATURETABLES,
  '__module__' : 'feature_catalog_pb2'
  # @@protoc_insertion_point(class_scope:featurestore.SearchFeatureTables)
  })
_sym_db.RegisterMessage(SearchFeatureTables)
_sym_db.RegisterMessage(SearchFeatureTables.Response)

GetFeatureTablesById = _reflection.GeneratedProtocolMessageType('GetFeatureTablesById', (_message.Message,), {

  'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
    'DESCRIPTOR' : _GETFEATURETABLESBYID_RESPONSE,
    '__module__' : 'feature_catalog_pb2'
    # @@protoc_insertion_point(class_scope:featurestore.GetFeatureTablesById.Response)
    })
  ,
  'DESCRIPTOR' : _GETFEATURETABLESBYID,
  '__module__' : 'feature_catalog_pb2'
  # @@protoc_insertion_point(class_scope:featurestore.GetFeatureTablesById)
  })
_sym_db.RegisterMessage(GetFeatureTablesById)
_sym_db.RegisterMessage(GetFeatureTablesById.Response)

AddDataSources = _reflection.GeneratedProtocolMessageType('AddDataSources', (_message.Message,), {

  'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
    'DESCRIPTOR' : _ADDDATASOURCES_RESPONSE,
    '__module__' : 'feature_catalog_pb2'
    # @@protoc_insertion_point(class_scope:featurestore.AddDataSources.Response)
    })
  ,
  'DESCRIPTOR' : _ADDDATASOURCES,
  '__module__' : 'feature_catalog_pb2'
  # @@protoc_insertion_point(class_scope:featurestore.AddDataSources)
  })
_sym_db.RegisterMessage(AddDataSources)
_sym_db.RegisterMessage(AddDataSources.Response)

DeleteDataSources = _reflection.GeneratedProtocolMessageType('DeleteDataSources', (_message.Message,), {

  'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
    'DESCRIPTOR' : _DELETEDATASOURCES_RESPONSE,
    '__module__' : 'feature_catalog_pb2'
    # @@protoc_insertion_point(class_scope:featurestore.DeleteDataSources.Response)
    })
  ,
  'DESCRIPTOR' : _DELETEDATASOURCES,
  '__module__' : 'feature_catalog_pb2'
  # @@protoc_insertion_point(class_scope:featurestore.DeleteDataSources)
  })
_sym_db.RegisterMessage(DeleteDataSources)
_sym_db.RegisterMessage(DeleteDataSources.Response)

PublishFeatureTable = _reflection.GeneratedProtocolMessageType('PublishFeatureTable', (_message.Message,), {

  'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
    'DESCRIPTOR' : _PUBLISHFEATURETABLE_RESPONSE,
    '__module__' : 'feature_catalog_pb2'
    # @@protoc_insertion_point(class_scope:featurestore.PublishFeatureTable.Response)
    })
  ,
  'DESCRIPTOR' : _PUBLISHFEATURETABLE,
  '__module__' : 'feature_catalog_pb2'
  # @@protoc_insertion_point(class_scope:featurestore.PublishFeatureTable)
  })
_sym_db.RegisterMessage(PublishFeatureTable)
_sym_db.RegisterMessage(PublishFeatureTable.Response)

GetFeatures = _reflection.GeneratedProtocolMessageType('GetFeatures', (_message.Message,), {

  'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
    'DESCRIPTOR' : _GETFEATURES_RESPONSE,
    '__module__' : 'feature_catalog_pb2'
    # @@protoc_insertion_point(class_scope:featurestore.GetFeatures.Response)
    })
  ,
  'DESCRIPTOR' : _GETFEATURES,
  '__module__' : 'feature_catalog_pb2'
  # @@protoc_insertion_point(class_scope:featurestore.GetFeatures)
  })
_sym_db.RegisterMessage(GetFeatures)
_sym_db.RegisterMessage(GetFeatures.Response)

GetFeature = _reflection.GeneratedProtocolMessageType('GetFeature', (_message.Message,), {

  'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
    'DESCRIPTOR' : _GETFEATURE_RESPONSE,
    '__module__' : 'feature_catalog_pb2'
    # @@protoc_insertion_point(class_scope:featurestore.GetFeature.Response)
    })
  ,
  'DESCRIPTOR' : _GETFEATURE,
  '__module__' : 'feature_catalog_pb2'
  # @@protoc_insertion_point(class_scope:featurestore.GetFeature)
  })
_sym_db.RegisterMessage(GetFeature)
_sym_db.RegisterMessage(GetFeature.Response)

CreateFeatures = _reflection.GeneratedProtocolMessageType('CreateFeatures', (_message.Message,), {

  'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
    'DESCRIPTOR' : _CREATEFEATURES_RESPONSE,
    '__module__' : 'feature_catalog_pb2'
    # @@protoc_insertion_point(class_scope:featurestore.CreateFeatures.Response)
    })
  ,
  'DESCRIPTOR' : _CREATEFEATURES,
  '__module__' : 'feature_catalog_pb2'
  # @@protoc_insertion_point(class_scope:featurestore.CreateFeatures)
  })
_sym_db.RegisterMessage(CreateFeatures)
_sym_db.RegisterMessage(CreateFeatures.Response)

UpdateFeature = _reflection.GeneratedProtocolMessageType('UpdateFeature', (_message.Message,), {

  'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
    'DESCRIPTOR' : _UPDATEFEATURE_RESPONSE,
    '__module__' : 'feature_catalog_pb2'
    # @@protoc_insertion_point(class_scope:featurestore.UpdateFeature.Response)
    })
  ,
  'DESCRIPTOR' : _UPDATEFEATURE,
  '__module__' : 'feature_catalog_pb2'
  # @@protoc_insertion_point(class_scope:featurestore.UpdateFeature)
  })
_sym_db.RegisterMessage(UpdateFeature)
_sym_db.RegisterMessage(UpdateFeature.Response)

DeleteFeature = _reflection.GeneratedProtocolMessageType('DeleteFeature', (_message.Message,), {

  'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
    'DESCRIPTOR' : _DELETEFEATURE_RESPONSE,
    '__module__' : 'feature_catalog_pb2'
    # @@protoc_insertion_point(class_scope:featurestore.DeleteFeature.Response)
    })
  ,
  'DESCRIPTOR' : _DELETEFEATURE,
  '__module__' : 'feature_catalog_pb2'
  # @@protoc_insertion_point(class_scope:featurestore.DeleteFeature)
  })
_sym_db.RegisterMessage(DeleteFeature)
_sym_db.RegisterMessage(DeleteFeature.Response)

AddProducer = _reflection.GeneratedProtocolMessageType('AddProducer', (_message.Message,), {

  'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
    'DESCRIPTOR' : _ADDPRODUCER_RESPONSE,
    '__module__' : 'feature_catalog_pb2'
    # @@protoc_insertion_point(class_scope:featurestore.AddProducer.Response)
    })
  ,
  'DESCRIPTOR' : _ADDPRODUCER,
  '__module__' : 'feature_catalog_pb2'
  # @@protoc_insertion_point(class_scope:featurestore.AddProducer)
  })
_sym_db.RegisterMessage(AddProducer)
_sym_db.RegisterMessage(AddProducer.Response)

AddConsumer = _reflection.GeneratedProtocolMessageType('AddConsumer', (_message.Message,), {

  'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
    'DESCRIPTOR' : _ADDCONSUMER_RESPONSE,
    '__module__' : 'feature_catalog_pb2'
    # @@protoc_insertion_point(class_scope:featurestore.AddConsumer.Response)
    })
  ,
  'DESCRIPTOR' : _ADDCONSUMER,
  '__module__' : 'feature_catalog_pb2'
  # @@protoc_insertion_point(class_scope:featurestore.AddConsumer)
  })
_sym_db.RegisterMessage(AddConsumer)
_sym_db.RegisterMessage(AddConsumer.Response)

GetConsumers = _reflection.GeneratedProtocolMessageType('GetConsumers', (_message.Message,), {

  'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
    'DESCRIPTOR' : _GETCONSUMERS_RESPONSE,
    '__module__' : 'feature_catalog_pb2'
    # @@protoc_insertion_point(class_scope:featurestore.GetConsumers.Response)
    })
  ,
  'DESCRIPTOR' : _GETCONSUMERS,
  '__module__' : 'feature_catalog_pb2'
  # @@protoc_insertion_point(class_scope:featurestore.GetConsumers)
  })
_sym_db.RegisterMessage(GetConsumers)
_sym_db.RegisterMessage(GetConsumers.Response)

GetFeatureStoreWidePermissions = _reflection.GeneratedProtocolMessageType('GetFeatureStoreWidePermissions', (_message.Message,), {

  'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
    'DESCRIPTOR' : _GETFEATURESTOREWIDEPERMISSIONS_RESPONSE,
    '__module__' : 'feature_catalog_pb2'
    # @@protoc_insertion_point(class_scope:featurestore.GetFeatureStoreWidePermissions.Response)
    })
  ,
  'DESCRIPTOR' : _GETFEATURESTOREWIDEPERMISSIONS,
  '__module__' : 'feature_catalog_pb2'
  # @@protoc_insertion_point(class_scope:featurestore.GetFeatureStoreWidePermissions)
  })
_sym_db.RegisterMessage(GetFeatureStoreWidePermissions)
_sym_db.RegisterMessage(GetFeatureStoreWidePermissions.Response)

GetOnlineStore = _reflection.GeneratedProtocolMessageType('GetOnlineStore', (_message.Message,), {

  'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
    'DESCRIPTOR' : _GETONLINESTORE_RESPONSE,
    '__module__' : 'feature_catalog_pb2'
    # @@protoc_insertion_point(class_scope:featurestore.GetOnlineStore.Response)
    })
  ,
  'DESCRIPTOR' : _GETONLINESTORE,
  '__module__' : 'feature_catalog_pb2'
  # @@protoc_insertion_point(class_scope:featurestore.GetOnlineStore)
  })
_sym_db.RegisterMessage(GetOnlineStore)
_sym_db.RegisterMessage(GetOnlineStore.Response)

GetModelServingMetadata = _reflection.GeneratedProtocolMessageType('GetModelServingMetadata', (_message.Message,), {

  'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
    'DESCRIPTOR' : _GETMODELSERVINGMETADATA_RESPONSE,
    '__module__' : 'feature_catalog_pb2'
    # @@protoc_insertion_point(class_scope:featurestore.GetModelServingMetadata.Response)
    })
  ,
  'DESCRIPTOR' : _GETMODELSERVINGMETADATA,
  '__module__' : 'feature_catalog_pb2'
  # @@protoc_insertion_point(class_scope:featurestore.GetModelServingMetadata)
  })
_sym_db.RegisterMessage(GetModelServingMetadata)
_sym_db.RegisterMessage(GetModelServingMetadata.Response)

SetTags = _reflection.GeneratedProtocolMessageType('SetTags', (_message.Message,), {

  'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
    'DESCRIPTOR' : _SETTAGS_RESPONSE,
    '__module__' : 'feature_catalog_pb2'
    # @@protoc_insertion_point(class_scope:featurestore.SetTags.Response)
    })
  ,
  'DESCRIPTOR' : _SETTAGS,
  '__module__' : 'feature_catalog_pb2'
  # @@protoc_insertion_point(class_scope:featurestore.SetTags)
  })
_sym_db.RegisterMessage(SetTags)
_sym_db.RegisterMessage(SetTags.Response)

GetTags = _reflection.GeneratedProtocolMessageType('GetTags', (_message.Message,), {

  'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
    'DESCRIPTOR' : _GETTAGS_RESPONSE,
    '__module__' : 'feature_catalog_pb2'
    # @@protoc_insertion_point(class_scope:featurestore.GetTags.Response)
    })
  ,
  'DESCRIPTOR' : _GETTAGS,
  '__module__' : 'feature_catalog_pb2'
  # @@protoc_insertion_point(class_scope:featurestore.GetTags)
  })
_sym_db.RegisterMessage(GetTags)
_sym_db.RegisterMessage(GetTags.Response)

DeleteTags = _reflection.GeneratedProtocolMessageType('DeleteTags', (_message.Message,), {

  'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
    'DESCRIPTOR' : _DELETETAGS_RESPONSE,
    '__module__' : 'feature_catalog_pb2'
    # @@protoc_insertion_point(class_scope:featurestore.DeleteTags.Response)
    })
  ,
  'DESCRIPTOR' : _DELETETAGS,
  '__module__' : 'feature_catalog_pb2'
  # @@protoc_insertion_point(class_scope:featurestore.DeleteTags)
  })
_sym_db.RegisterMessage(DeleteTags)
_sym_db.RegisterMessage(DeleteTags.Response)

KeySpec = _reflection.GeneratedProtocolMessageType('KeySpec', (_message.Message,), {
  'DESCRIPTOR' : _KEYSPEC,
  '__module__' : 'feature_catalog_pb2'
  # @@protoc_insertion_point(class_scope:featurestore.KeySpec)
  })
_sym_db.RegisterMessage(KeySpec)

FeatureTable = _reflection.GeneratedProtocolMessageType('FeatureTable', (_message.Message,), {
  'DESCRIPTOR' : _FEATURETABLE,
  '__module__' : 'feature_catalog_pb2'
  # @@protoc_insertion_point(class_scope:featurestore.FeatureTable)
  })
_sym_db.RegisterMessage(FeatureTable)

Feature = _reflection.GeneratedProtocolMessageType('Feature', (_message.Message,), {
  'DESCRIPTOR' : _FEATURE,
  '__module__' : 'feature_catalog_pb2'
  # @@protoc_insertion_point(class_scope:featurestore.Feature)
  })
_sym_db.RegisterMessage(Feature)

DataSource = _reflection.GeneratedProtocolMessageType('DataSource', (_message.Message,), {
  'DESCRIPTOR' : _DATASOURCE,
  '__module__' : 'feature_catalog_pb2'
  # @@protoc_insertion_point(class_scope:featurestore.DataSource)
  })
_sym_db.RegisterMessage(DataSource)

ConsumedFeatures = _reflection.GeneratedProtocolMessageType('ConsumedFeatures', (_message.Message,), {
  'DESCRIPTOR' : _CONSUMEDFEATURES,
  '__module__' : 'feature_catalog_pb2'
  # @@protoc_insertion_point(class_scope:featurestore.ConsumedFeatures)
  })
_sym_db.RegisterMessage(ConsumedFeatures)

Notebook = _reflection.GeneratedProtocolMessageType('Notebook', (_message.Message,), {
  'DESCRIPTOR' : _NOTEBOOK,
  '__module__' : 'feature_catalog_pb2'
  # @@protoc_insertion_point(class_scope:featurestore.Notebook)
  })
_sym_db.RegisterMessage(Notebook)

Job = _reflection.GeneratedProtocolMessageType('Job', (_message.Message,), {
  'DESCRIPTOR' : _JOB,
  '__module__' : 'feature_catalog_pb2'
  # @@protoc_insertion_point(class_scope:featurestore.Job)
  })
_sym_db.RegisterMessage(Job)

DltPipeline = _reflection.GeneratedProtocolMessageType('DltPipeline', (_message.Message,), {
  'DESCRIPTOR' : _DLTPIPELINE,
  '__module__' : 'feature_catalog_pb2'
  # @@protoc_insertion_point(class_scope:featurestore.DltPipeline)
  })
_sym_db.RegisterMessage(DltPipeline)

Consumer = _reflection.GeneratedProtocolMessageType('Consumer', (_message.Message,), {
  'DESCRIPTOR' : _CONSUMER,
  '__module__' : 'feature_catalog_pb2'
  # @@protoc_insertion_point(class_scope:featurestore.Consumer)
  })
_sym_db.RegisterMessage(Consumer)

FeatureTableFeatures = _reflection.GeneratedProtocolMessageType('FeatureTableFeatures', (_message.Message,), {
  'DESCRIPTOR' : _FEATURETABLEFEATURES,
  '__module__' : 'feature_catalog_pb2'
  # @@protoc_insertion_point(class_scope:featurestore.FeatureTableFeatures)
  })
_sym_db.RegisterMessage(FeatureTableFeatures)

Tag = _reflection.GeneratedProtocolMessageType('Tag', (_message.Message,), {
  'DESCRIPTOR' : _TAG,
  '__module__' : 'feature_catalog_pb2'
  # @@protoc_insertion_point(class_scope:featurestore.Tag)
  })
_sym_db.RegisterMessage(Tag)

MonitoringConfig = _reflection.GeneratedProtocolMessageType('MonitoringConfig', (_message.Message,), {
  'DESCRIPTOR' : _MONITORINGCONFIG,
  '__module__' : 'feature_catalog_pb2'
  # @@protoc_insertion_point(class_scope:featurestore.MonitoringConfig)
  })
_sym_db.RegisterMessage(MonitoringConfig)

Profile = _reflection.GeneratedProtocolMessageType('Profile', (_message.Message,), {
  'DESCRIPTOR' : _PROFILE,
  '__module__' : 'feature_catalog_pb2'
  # @@protoc_insertion_point(class_scope:featurestore.Profile)
  })
_sym_db.RegisterMessage(Profile)

TableStats = _reflection.GeneratedProtocolMessageType('TableStats', (_message.Message,), {
  'DESCRIPTOR' : _TABLESTATS,
  '__module__' : 'feature_catalog_pb2'
  # @@protoc_insertion_point(class_scope:featurestore.TableStats)
  })
_sym_db.RegisterMessage(TableStats)

SetMonitoringConfig = _reflection.GeneratedProtocolMessageType('SetMonitoringConfig', (_message.Message,), {

  'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
    'DESCRIPTOR' : _SETMONITORINGCONFIG_RESPONSE,
    '__module__' : 'feature_catalog_pb2'
    # @@protoc_insertion_point(class_scope:featurestore.SetMonitoringConfig.Response)
    })
  ,
  'DESCRIPTOR' : _SETMONITORINGCONFIG,
  '__module__' : 'feature_catalog_pb2'
  # @@protoc_insertion_point(class_scope:featurestore.SetMonitoringConfig)
  })
_sym_db.RegisterMessage(SetMonitoringConfig)
_sym_db.RegisterMessage(SetMonitoringConfig.Response)

GetMonitoringConfig = _reflection.GeneratedProtocolMessageType('GetMonitoringConfig', (_message.Message,), {

  'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
    'DESCRIPTOR' : _GETMONITORINGCONFIG_RESPONSE,
    '__module__' : 'feature_catalog_pb2'
    # @@protoc_insertion_point(class_scope:featurestore.GetMonitoringConfig.Response)
    })
  ,
  'DESCRIPTOR' : _GETMONITORINGCONFIG,
  '__module__' : 'feature_catalog_pb2'
  # @@protoc_insertion_point(class_scope:featurestore.GetMonitoringConfig)
  })
_sym_db.RegisterMessage(GetMonitoringConfig)
_sym_db.RegisterMessage(GetMonitoringConfig.Response)

ListProfiles = _reflection.GeneratedProtocolMessageType('ListProfiles', (_message.Message,), {

  'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
    'DESCRIPTOR' : _LISTPROFILES_RESPONSE,
    '__module__' : 'feature_catalog_pb2'
    # @@protoc_insertion_point(class_scope:featurestore.ListProfiles.Response)
    })
  ,
  'DESCRIPTOR' : _LISTPROFILES,
  '__module__' : 'feature_catalog_pb2'
  # @@protoc_insertion_point(class_scope:featurestore.ListProfiles)
  })
_sym_db.RegisterMessage(ListProfiles)
_sym_db.RegisterMessage(ListProfiles.Response)

GetProfile = _reflection.GeneratedProtocolMessageType('GetProfile', (_message.Message,), {

  'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
    'DESCRIPTOR' : _GETPROFILE_RESPONSE,
    '__module__' : 'feature_catalog_pb2'
    # @@protoc_insertion_point(class_scope:featurestore.GetProfile.Response)
    })
  ,
  'DESCRIPTOR' : _GETPROFILE,
  '__module__' : 'feature_catalog_pb2'
  # @@protoc_insertion_point(class_scope:featurestore.GetProfile)
  })
_sym_db.RegisterMessage(GetProfile)
_sym_db.RegisterMessage(GetProfile.Response)

CreateProfile = _reflection.GeneratedProtocolMessageType('CreateProfile', (_message.Message,), {

  'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
    'DESCRIPTOR' : _CREATEPROFILE_RESPONSE,
    '__module__' : 'feature_catalog_pb2'
    # @@protoc_insertion_point(class_scope:featurestore.CreateProfile.Response)
    })
  ,
  'DESCRIPTOR' : _CREATEPROFILE,
  '__module__' : 'feature_catalog_pb2'
  # @@protoc_insertion_point(class_scope:featurestore.CreateProfile)
  })
_sym_db.RegisterMessage(CreateProfile)
_sym_db.RegisterMessage(CreateProfile.Response)

DeleteProfiles = _reflection.GeneratedProtocolMessageType('DeleteProfiles', (_message.Message,), {

  'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
    'DESCRIPTOR' : _DELETEPROFILES_RESPONSE,
    '__module__' : 'feature_catalog_pb2'
    # @@protoc_insertion_point(class_scope:featurestore.DeleteProfiles.Response)
    })
  ,
  'DESCRIPTOR' : _DELETEPROFILES,
  '__module__' : 'feature_catalog_pb2'
  # @@protoc_insertion_point(class_scope:featurestore.DeleteProfiles)
  })
_sym_db.RegisterMessage(DeleteProfiles)
_sym_db.RegisterMessage(DeleteProfiles.Response)

_FEATURESTORESERVICE = DESCRIPTOR.services_by_name['FeatureStoreService']
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n%com.databricks.api.proto.featurestore\220\001\001\240\001\001\342?3\020\001\032/com.databricks.featurestore.api.FeatureStoreRPC'
  _HEALTHCHECK._options = None
  _HEALTHCHECK._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
  _CREATEFEATURETABLE.fields_by_name['name']._options = None
  _CREATEFEATURETABLE.fields_by_name['name']._serialized_options = b'\370\206\031\001'
  _CREATEFEATURETABLE._options = None
  _CREATEFEATURETABLE._serialized_options = b'\342?!\n\037FeatureStoreRPC[$this.Response]'
  _UPDATEFEATURETABLE.fields_by_name['name']._options = None
  _UPDATEFEATURETABLE.fields_by_name['name']._serialized_options = b'\370\206\031\001'
  _UPDATEFEATURETABLE._options = None
  _UPDATEFEATURETABLE._serialized_options = b'\342?!\n\037FeatureStoreRPC[$this.Response]'
  _DELETEFEATURETABLE.fields_by_name['name']._options = None
  _DELETEFEATURETABLE.fields_by_name['name']._serialized_options = b'\370\206\031\001'
  _DELETEFEATURETABLE._options = None
  _DELETEFEATURETABLE._serialized_options = b'\342?!\n\037FeatureStoreRPC[$this.Response]'
  _GETFEATURETABLE.fields_by_name['name']._options = None
  _GETFEATURETABLE.fields_by_name['name']._serialized_options = b'\370\206\031\001'
  _GETFEATURETABLE._options = None
  _GETFEATURETABLE._serialized_options = b'\342?!\n\037FeatureStoreRPC[$this.Response]'
  _SEARCHFEATURETABLES._options = None
  _SEARCHFEATURETABLES._serialized_options = b'\342?!\n\037FeatureStoreRPC[$this.Response]'
  _GETFEATURETABLESBYID._options = None
  _GETFEATURETABLESBYID._serialized_options = b'\342?!\n\037FeatureStoreRPC[$this.Response]'
  _ADDDATASOURCES.fields_by_name['feature_table']._options = None
  _ADDDATASOURCES.fields_by_name['feature_table']._serialized_options = b'\370\206\031\001'
  _ADDDATASOURCES._options = None
  _ADDDATASOURCES._serialized_options = b'\342?!\n\037FeatureStoreRPC[$this.Response]'
  _DELETEDATASOURCES.fields_by_name['feature_table']._options = None
  _DELETEDATASOURCES.fields_by_name['feature_table']._serialized_options = b'\370\206\031\001'
  _DELETEDATASOURCES._options = None
  _DELETEDATASOURCES._serialized_options = b'\342?!\n\037FeatureStoreRPC[$this.Response]'
  _PUBLISHFEATURETABLE.fields_by_name['feature_table']._options = None
  _PUBLISHFEATURETABLE.fields_by_name['feature_table']._serialized_options = b'\370\206\031\001'
  _PUBLISHFEATURETABLE.fields_by_name['online_table']._options = None
  _PUBLISHFEATURETABLE.fields_by_name['online_table']._serialized_options = b'\370\206\031\001'
  _PUBLISHFEATURETABLE.fields_by_name['cloud']._options = None
  _PUBLISHFEATURETABLE.fields_by_name['cloud']._serialized_options = b'\370\206\031\001'
  _PUBLISHFEATURETABLE.fields_by_name['store_type']._options = None
  _PUBLISHFEATURETABLE.fields_by_name['store_type']._serialized_options = b'\370\206\031\001'
  _PUBLISHFEATURETABLE._options = None
  _PUBLISHFEATURETABLE._serialized_options = b'\342?!\n\037FeatureStoreRPC[$this.Response]'
  _GETFEATURES.fields_by_name['feature_table']._options = None
  _GETFEATURES.fields_by_name['feature_table']._serialized_options = b'\370\206\031\001'
  _GETFEATURES._options = None
  _GETFEATURES._serialized_options = b'\342?!\n\037FeatureStoreRPC[$this.Response]'
  _GETFEATURE.fields_by_name['feature_table']._options = None
  _GETFEATURE.fields_by_name['feature_table']._serialized_options = b'\370\206\031\001'
  _GETFEATURE.fields_by_name['name']._options = None
  _GETFEATURE.fields_by_name['name']._serialized_options = b'\370\206\031\001'
  _GETFEATURE._options = None
  _GETFEATURE._serialized_options = b'\342?!\n\037FeatureStoreRPC[$this.Response]'
  _CREATEFEATURES.fields_by_name['feature_table']._options = None
  _CREATEFEATURES.fields_by_name['feature_table']._serialized_options = b'\370\206\031\001'
  _CREATEFEATURES._options = None
  _CREATEFEATURES._serialized_options = b'\342?!\n\037FeatureStoreRPC[$this.Response]'
  _UPDATEFEATURE.fields_by_name['feature_table']._options = None
  _UPDATEFEATURE.fields_by_name['feature_table']._serialized_options = b'\370\206\031\001'
  _UPDATEFEATURE.fields_by_name['name']._options = None
  _UPDATEFEATURE.fields_by_name['name']._serialized_options = b'\370\206\031\001'
  _UPDATEFEATURE._options = None
  _UPDATEFEATURE._serialized_options = b'\342?!\n\037FeatureStoreRPC[$this.Response]'
  _DELETEFEATURE.fields_by_name['feature_table']._options = None
  _DELETEFEATURE.fields_by_name['feature_table']._serialized_options = b'\370\206\031\001'
  _DELETEFEATURE.fields_by_name['name']._options = None
  _DELETEFEATURE.fields_by_name['name']._serialized_options = b'\370\206\031\001'
  _DELETEFEATURE._options = None
  _DELETEFEATURE._serialized_options = b'\342?!\n\037FeatureStoreRPC[$this.Response]'
  _ADDPRODUCER.fields_by_name['feature_table']._options = None
  _ADDPRODUCER.fields_by_name['feature_table']._serialized_options = b'\370\206\031\001'
  _ADDPRODUCER._options = None
  _ADDPRODUCER._serialized_options = b'\342?!\n\037FeatureStoreRPC[$this.Response]'
  _ADDCONSUMER._options = None
  _ADDCONSUMER._serialized_options = b'\342?!\n\037FeatureStoreRPC[$this.Response]'
  _GETCONSUMERS.fields_by_name['feature_table']._options = None
  _GETCONSUMERS.fields_by_name['feature_table']._serialized_options = b'\370\206\031\001'
  _GETCONSUMERS._options = None
  _GETCONSUMERS._serialized_options = b'\342?!\n\037FeatureStoreRPC[$this.Response]'
  _GETFEATURESTOREWIDEPERMISSIONS._options = None
  _GETFEATURESTOREWIDEPERMISSIONS._serialized_options = b'\342?!\n\037FeatureStoreRPC[$this.Response]'
  _GETONLINESTORE.fields_by_name['feature_table']._options = None
  _GETONLINESTORE.fields_by_name['feature_table']._serialized_options = b'\370\206\031\001'
  _GETONLINESTORE.fields_by_name['online_table']._options = None
  _GETONLINESTORE.fields_by_name['online_table']._serialized_options = b'\370\206\031\001'
  _GETONLINESTORE.fields_by_name['cloud']._options = None
  _GETONLINESTORE.fields_by_name['cloud']._serialized_options = b'\370\206\031\001'
  _GETONLINESTORE.fields_by_name['store_type']._options = None
  _GETONLINESTORE.fields_by_name['store_type']._serialized_options = b'\370\206\031\001'
  _GETONLINESTORE._options = None
  _GETONLINESTORE._serialized_options = b'\342?!\n\037FeatureStoreRPC[$this.Response]'
  _GETMODELSERVINGMETADATA._options = None
  _GETMODELSERVINGMETADATA._serialized_options = b'\342?!\n\037FeatureStoreRPC[$this.Response]'
  _SETTAGS._options = None
  _SETTAGS._serialized_options = b'\342?!\n\037FeatureStoreRPC[$this.Response]'
  _GETTAGS._options = None
  _GETTAGS._serialized_options = b'\342?!\n\037FeatureStoreRPC[$this.Response]'
  _DELETETAGS._options = None
  _DELETETAGS._serialized_options = b'\342?!\n\037FeatureStoreRPC[$this.Response]'
  _KEYSPEC.fields_by_name['name']._options = None
  _KEYSPEC.fields_by_name['name']._serialized_options = b'\370\206\031\001'
  _KEYSPEC.fields_by_name['data_type']._options = None
  _KEYSPEC.fields_by_name['data_type']._serialized_options = b'\370\206\031\001'
  _NOTEBOOK.fields_by_name['workspace_path']._options = None
  _NOTEBOOK.fields_by_name['workspace_path']._serialized_options = b'\030\001'
  _JOB.fields_by_name['job_name']._options = None
  _JOB.fields_by_name['job_name']._serialized_options = b'\030\001'
  _MONITORINGCONFIG.fields_by_name['feature_table']._options = None
  _MONITORINGCONFIG.fields_by_name['feature_table']._serialized_options = b'\370\206\031\001'
  _SETMONITORINGCONFIG.fields_by_name['feature_table']._options = None
  _SETMONITORINGCONFIG.fields_by_name['feature_table']._serialized_options = b'\370\206\031\001'
  _SETMONITORINGCONFIG._options = None
  _SETMONITORINGCONFIG._serialized_options = b'\342?!\n\037FeatureStoreRPC[$this.Response]'
  _GETMONITORINGCONFIG.fields_by_name['feature_table']._options = None
  _GETMONITORINGCONFIG.fields_by_name['feature_table']._serialized_options = b'\370\206\031\001'
  _GETMONITORINGCONFIG._options = None
  _GETMONITORINGCONFIG._serialized_options = b'\342?!\n\037FeatureStoreRPC[$this.Response]'
  _LISTPROFILES.fields_by_name['feature_table']._options = None
  _LISTPROFILES.fields_by_name['feature_table']._serialized_options = b'\370\206\031\001'
  _LISTPROFILES._options = None
  _LISTPROFILES._serialized_options = b'\342?!\n\037FeatureStoreRPC[$this.Response]'
  _GETPROFILE.fields_by_name['feature_table']._options = None
  _GETPROFILE.fields_by_name['feature_table']._serialized_options = b'\370\206\031\001'
  _GETPROFILE._options = None
  _GETPROFILE._serialized_options = b'\342?!\n\037FeatureStoreRPC[$this.Response]'
  _CREATEPROFILE._options = None
  _CREATEPROFILE._serialized_options = b'\342?!\n\037FeatureStoreRPC[$this.Response]'
  _DELETEPROFILES._options = None
  _DELETEPROFILES._serialized_options = b'\342?!\n\037FeatureStoreRPC[$this.Response]'
  _FEATURESTORESERVICE.methods_by_name['healthCheck']._options = None
  _FEATURESTORESERVICE.methods_by_name['healthCheck']._serialized_options = b'\362\206\031Y\n(\n\003GET\022\033/feature-store/health-check\032\004\010\002\020\000\020\003*+Health check for Feature Store service pod.'
  _FEATURESTORESERVICE.methods_by_name['createFeatureTable']._options = None
  _FEATURESTORESERVICE.methods_by_name['createFeatureTable']._serialized_options = b'\362\206\031M\n2\n\004POST\022$/feature-store/feature-tables/create\032\004\010\002\020\000\020\003*\025Create Feature Table.'
  _FEATURESTORESERVICE.methods_by_name['updateFeatureTable']._options = None
  _FEATURESTORESERVICE.methods_by_name['updateFeatureTable']._serialized_options = b'\362\206\031N\n3\n\005PATCH\022$/feature-store/feature-tables/update\032\004\010\002\020\000\020\003*\025Update Feature Table.'
  _FEATURESTORESERVICE.methods_by_name['deleteFeatureTable']._options = None
  _FEATURESTORESERVICE.methods_by_name['deleteFeatureTable']._serialized_options = b'\362\206\031O\n4\n\006DELETE\022$/feature-store/feature-tables/delete\032\004\010\002\020\000\020\003*\025Delete Feature Table.'
  _FEATURESTORESERVICE.methods_by_name['getFeatureTable']._options = None
  _FEATURESTORESERVICE.methods_by_name['getFeatureTable']._serialized_options = b'\362\206\031F\n.\n\003GET\022!/feature-store/feature-tables/get\032\004\010\002\020\000\020\003*\022Get Feature Table.'
  _FEATURESTORESERVICE.methods_by_name['searchFeatureTables']._options = None
  _FEATURESTORESERVICE.methods_by_name['searchFeatureTables']._serialized_options = b'\362\206\031\201\001\n1\n\003GET\022$/feature-store/feature-tables/search\032\004\010\002\020\000\n2\n\004POST\022$/feature-store/feature-tables/search\032\004\010\002\020\000\020\003*\026Search Feature Tables.'
  _FEATURESTORESERVICE.methods_by_name['getFeatureTablesById']._options = None
  _FEATURESTORESERVICE.methods_by_name['getFeatureTablesById']._serialized_options = b'\362\206\031U\n6\n\004POST\022(/feature-store/feature-tables/get-by-ids\032\004\010\002\020\000\020\003*\031Get Feature Tables by ID.'
  _FEATURESTORESERVICE.methods_by_name['addDataSources']._options = None
  _FEATURESTORESERVICE.methods_by_name['addDataSources']._serialized_options = b'\362\206\031e\n<\n\004POST\022./feature-store/feature-tables/add-data-sources\032\004\010\002\020\000\020\003*#Add Data Sources for Feature Table.'
  _FEATURESTORESERVICE.methods_by_name['deleteDataSources']._options = None
  _FEATURESTORESERVICE.methods_by_name['deleteDataSources']._serialized_options = b'\362\206\031k\n?\n\004POST\0221/feature-store/feature-tables/delete-data-sources\032\004\010\002\020\000\020\003*&Delete Data Sources for Feature Table.'
  _FEATURESTORESERVICE.methods_by_name['publishFeatureTable']._options = None
  _FEATURESTORESERVICE.methods_by_name['publishFeatureTable']._serialized_options = b'\362\206\031O\n3\n\004POST\022%/feature-store/feature-tables/publish\032\004\010\002\020\000\020\003*\026Publish Feature Table.'
  _FEATURESTORESERVICE.methods_by_name['getFeatures']._options = None
  _FEATURESTORESERVICE.methods_by_name['getFeatures']._serialized_options = b'\362\206\031;\n(\n\003GET\022\033/feature-store/features/get\032\004\010\002\020\000\020\003*\rGet Features.'
  _FEATURESTORESERVICE.methods_by_name['getFeature']._options = None
  _FEATURESTORESERVICE.methods_by_name['getFeature']._serialized_options = b'\362\206\031B\n0\n\003GET\022#/feature-store/features/get-by-name\032\004\010\002\020\000\020\003*\014Get Feature.'
  _FEATURESTORESERVICE.methods_by_name['createFeatures']._options = None
  _FEATURESTORESERVICE.methods_by_name['createFeatures']._serialized_options = b'\362\206\031B\n,\n\004POST\022\036/feature-store/features/create\032\004\010\002\020\000\020\003*\020Create Features.'
  _FEATURESTORESERVICE.methods_by_name['updateFeature']._options = None
  _FEATURESTORESERVICE.methods_by_name['updateFeature']._serialized_options = b'\362\206\031B\n-\n\005PATCH\022\036/feature-store/features/update\032\004\010\002\020\000\020\003*\017Update Feature.'
  _FEATURESTORESERVICE.methods_by_name['deleteFeature']._options = None
  _FEATURESTORESERVICE.methods_by_name['deleteFeature']._serialized_options = b'\362\206\031C\n.\n\006DELETE\022\036/feature-store/features/delete\032\004\010\002\020\000\020\003*\017Delete Feature.'
  _FEATURESTORESERVICE.methods_by_name['addProducer']._options = None
  _FEATURESTORESERVICE.methods_by_name['addProducer']._serialized_options = b'\362\206\031K\n8\n\004POST\022*/feature-store/feature-tables/add-producer\032\004\010\002\020\000\020\003*\rAdd Producer.'
  _FEATURESTORESERVICE.methods_by_name['addConsumer']._options = None
  _FEATURESTORESERVICE.methods_by_name['addConsumer']._serialized_options = b'\362\206\031E\n2\n\004POST\022$/feature-store/features/add-consumer\032\004\010\002\020\000\020\003*\rAdd Consumer.'
  _FEATURESTORESERVICE.methods_by_name['getConsumers']._options = None
  _FEATURESTORESERVICE.methods_by_name['getConsumers']._serialized_options = b'\362\206\031L\n8\n\003GET\022+/feature-store/feature-tables/get-consumers\032\004\010\002\020\000\020\003*\016Get Consumers.'
  _FEATURESTORESERVICE.methods_by_name['getFeatureStoreWidePermissions']._options = None
  _FEATURESTORESERVICE.methods_by_name['getFeatureStoreWidePermissions']._serialized_options = b'\362\206\031\220\001\nM\n\003GET\022@/feature-store/feature-tables/get-feature-store-wide-permissions\032\004\010\002\020\000\020\003*=Get Feature Store Wide Permission Level for the Current User.'
  _FEATURESTORESERVICE.methods_by_name['getOnlineStore']._options = None
  _FEATURESTORESERVICE.methods_by_name['getOnlineStore']._serialized_options = b'\362\206\031R\n;\n\003GET\022./feature-store/feature-tables/get-online-store\032\004\010\002\020\000\020\003*\021Get Online Store.'
  _FEATURESTORESERVICE.methods_by_name['getModelServingMetadata']._options = None
  _FEATURESTORESERVICE.methods_by_name['getModelServingMetadata']._serialized_options = b'\362\206\031\220\001\n7\n\004POST\022)/feature-store/model-serving/get-metadata\032\004\010\002\020\000\n6\n\003GET\022)/feature-store/model-serving/get-metadata\032\004\010\002\020\000\020\003*\033Get Model Serving Metadata.'
  _FEATURESTORESERVICE.methods_by_name['setTags']._options = None
  _FEATURESTORESERVICE.methods_by_name['setTags']._serialized_options = b'\362\206\0314\n%\n\004POST\022\027/feature-store/tags/set\032\004\010\002\020\000\020\003*\tSet Tags.'
  _FEATURESTORESERVICE.methods_by_name['getTags']._options = None
  _FEATURESTORESERVICE.methods_by_name['getTags']._serialized_options = b'\362\206\0313\n$\n\003GET\022\027/feature-store/tags/get\032\004\010\002\020\000\020\003*\tGet Tags.'
  _FEATURESTORESERVICE.methods_by_name['deleteTags']._options = None
  _FEATURESTORESERVICE.methods_by_name['deleteTags']._serialized_options = b'\362\206\031<\n*\n\006DELETE\022\032/feature-store/tags/delete\032\004\010\002\020\000\020\003*\014Delete Tags.'
  _FEATURESTORESERVICE.methods_by_name['setMonitoringConfig']._options = None
  _FEATURESTORESERVICE.methods_by_name['setMonitoringConfig']._serialized_options = b'\362\206\031W\n;\n\004POST\0223/feature-store/feature-tables/set-monitoring-config\020\003*\026Set Monitoring Config.'
  _FEATURESTORESERVICE.methods_by_name['getMonitoringConfig']._options = None
  _FEATURESTORESERVICE.methods_by_name['getMonitoringConfig']._serialized_options = b'\362\206\031V\n:\n\003GET\0223/feature-store/feature-tables/get-monitoring-config\020\003*\026Get Monitoring Config.'
  _FEATURESTORESERVICE.methods_by_name['listProfiles']._options = None
  _FEATURESTORESERVICE.methods_by_name['listProfiles']._serialized_options = b'\362\206\031_\n#\n\003GET\022\034/feature-store/profiles/list\020\003*6List all the profiles that belongs to a feature table.'
  _FEATURESTORESERVICE.methods_by_name['getProfile']._options = None
  _FEATURESTORESERVICE.methods_by_name['getProfile']._serialized_options = b'\362\206\031S\n\"\n\003GET\022\033/feature-store/profiles/get\020\003*+Fetch the data profile for a feature table.'
  _FEATURESTORESERVICE.methods_by_name['createProfile']._options = None
  _FEATURESTORESERVICE.methods_by_name['createProfile']._serialized_options = b'\362\206\031T\n&\n\004POST\022\036/feature-store/profiles/create\020\003*(Save a data profile for a feature table.'
  _FEATURESTORESERVICE.methods_by_name['deleteProfiles']._options = None
  _FEATURESTORESERVICE.methods_by_name['deleteProfiles']._serialized_options = b'\362\206\031^\n+\n\006DELETE\022!/feature-store/profiles/deleteAll\020\003*-Delete all data profiles for a feature table.'
  _PERMISSIONLEVEL._serialized_start=8815
  _PERMISSIONLEVEL._serialized_end=8910
  _PRODUCERACTION._serialized_start=8912
  _PRODUCERACTION._serialized_end=8965
  _ASSOCIATIONTYPE._serialized_start=8967
  _ASSOCIATIONTYPE._serialized_end=9011
  _SEARCHSCOPE._serialized_start=9013
  _SEARCHSCOPE._serialized_end=9120
  _REFRESHMODE._serialized_start=9122
  _REFRESHMODE._serialized_end=9173
  _HEALTHCHECK._serialized_start=110
  _HEALTHCHECK._serialized_end=260
  _HEALTHCHECK_RESPONSE._serialized_start=170
  _HEALTHCHECK_RESPONSE._serialized_end=215
  _CREATEFEATURETABLE._serialized_start=263
  _CREATEFEATURETABLE._serialized_end=629
  _CREATEFEATURETABLE_RESPONSE._serialized_start=530
  _CREATEFEATURETABLE_RESPONSE._serialized_end=591
  _UPDATEFEATURETABLE._serialized_start=632
  _UPDATEFEATURETABLE._serialized_end=794
  _UPDATEFEATURETABLE_RESPONSE._serialized_start=530
  _UPDATEFEATURETABLE_RESPONSE._serialized_end=591
  _DELETEFEATURETABLE._serialized_start=796
  _DELETEFEATURETABLE._serialized_end=910
  _DELETEFEATURETABLE_RESPONSE._serialized_start=170
  _DELETEFEATURETABLE_RESPONSE._serialized_end=180
  _GETFEATURETABLE._serialized_start=913
  _GETFEATURETABLE._serialized_end=1051
  _GETFEATURETABLE_RESPONSE._serialized_start=530
  _GETFEATURETABLE_RESPONSE._serialized_end=591
  _SEARCHFEATURETABLES._serialized_start=1054
  _SEARCHFEATURETABLES._serialized_end=1333
  _SEARCHFEATURETABLES_RESPONSE._serialized_start=1208
  _SEARCHFEATURETABLES_RESPONSE._serialized_end=1295
  _GETFEATURETABLESBYID._serialized_start=1336
  _GETFEATURETABLESBYID._serialized_end=1473
  _GETFEATURETABLESBYID_RESPONSE._serialized_start=1208
  _GETFEATURETABLESBYID_RESPONSE._serialized_end=1270
  _ADDDATASOURCES._serialized_start=1476
  _ADDDATASOURCES._serialized_end=1694
  _ADDDATASOURCES_RESPONSE._serialized_start=170
  _ADDDATASOURCES_RESPONSE._serialized_end=180
  _DELETEDATASOURCES._serialized_start=1696
  _DELETEDATASOURCES._serialized_end=1811
  _DELETEDATASOURCES_RESPONSE._serialized_start=170
  _DELETEDATASOURCES_RESPONSE._serialized_end=180
  _PUBLISHFEATURETABLE._serialized_start=1814
  _PUBLISHFEATURETABLE._serialized_end=2501
  _PUBLISHFEATURETABLE_RESPONSE._serialized_start=2367
  _PUBLISHFEATURETABLE_RESPONSE._serialized_end=2440
  _GETFEATURES._serialized_start=2504
  _GETFEATURES._serialized_end=2708
  _GETFEATURES_RESPONSE._serialized_start=2594
  _GETFEATURES_RESPONSE._serialized_end=2670
  _GETFEATURE._serialized_start=2711
  _GETFEATURE._serialized_end=2862
  _GETFEATURE_RESPONSE._serialized_start=2774
  _GETFEATURE_RESPONSE._serialized_end=2824
  _CREATEFEATURES._serialized_start=2865
  _CREATEFEATURES._serialized_end=3001
  _CREATEFEATURES_RESPONSE._serialized_start=170
  _CREATEFEATURES_RESPONSE._serialized_end=180
  _UPDATEFEATURE._serialized_start=3004
  _UPDATEFEATURE._serialized_end=3225
  _UPDATEFEATURE_RESPONSE._serialized_start=2774
  _UPDATEFEATURE_RESPONSE._serialized_end=2824
  _DELETEFEATURE._serialized_start=3227
  _DELETEFEATURE._serialized_end=3341
  _DELETEFEATURE_RESPONSE._serialized_start=170
  _DELETEFEATURE_RESPONSE._serialized_end=180
  _ADDPRODUCER._serialized_start=3344
  _ADDPRODUCER._serialized_end=3636
  _ADDPRODUCER_RESPONSE._serialized_start=170
  _ADDPRODUCER_RESPONSE._serialized_end=180
  _ADDCONSUMER._serialized_start=3639
  _ADDCONSUMER._serialized_end=3897
  _ADDCONSUMER_RESPONSE._serialized_start=170
  _ADDCONSUMER_RESPONSE._serialized_end=180
  _GETCONSUMERS._serialized_start=3900
  _GETCONSUMERS._serialized_end=4036
  _GETCONSUMERS_RESPONSE._serialized_start=3945
  _GETCONSUMERS_RESPONSE._serialized_end=3998
  _GETFEATURESTOREWIDEPERMISSIONS._serialized_start=4039
  _GETFEATURESTOREWIDEPERMISSIONS._serialized_end=4178
  _GETFEATURESTOREWIDEPERMISSIONS_RESPONSE._serialized_start=4073
  _GETFEATURESTOREWIDEPERMISSIONS_RESPONSE._serialized_end=4140
  _GETONLINESTORE._serialized_start=4181
  _GETONLINESTORE._serialized_end=4548
  _GETONLINESTORE_RESPONSE._serialized_start=2367
  _GETONLINESTORE_RESPONSE._serialized_end=2440
  _GETMODELSERVINGMETADATA._serialized_start=4551
  _GETMODELSERVINGMETADATA._serialized_end=4765
  _GETMODELSERVINGMETADATA_RESPONSE._serialized_start=4646
  _GETMODELSERVINGMETADATA_RESPONSE._serialized_end=4727
  _SETTAGS._serialized_start=4768
  _SETTAGS._serialized_end=4923
  _SETTAGS_RESPONSE._serialized_start=170
  _SETTAGS_RESPONSE._serialized_end=180
  _GETTAGS._serialized_start=4926
  _GETTAGS._serialized_end=5081
  _GETTAGS_RESPONSE._serialized_start=4987
  _GETTAGS_RESPONSE._serialized_end=5030
  _DELETETAGS._serialized_start=5084
  _DELETETAGS._serialized_end=5223
  _DELETETAGS_RESPONSE._serialized_start=170
  _DELETETAGS_RESPONSE._serialized_end=180
  _KEYSPEC._serialized_start=5225
  _KEYSPEC._serialized_end=5306
  _FEATURETABLE._serialized_start=5309
  _FEATURETABLE._serialized_end=5902
  _FEATURE._serialized_start=5905
  _FEATURE._serialized_end=6161
  _DATASOURCE._serialized_start=6163
  _DATASOURCE._serialized_end=6227
  _CONSUMEDFEATURES._serialized_start=6229
  _CONSUMEDFEATURES._serialized_end=6277
  _NOTEBOOK._serialized_start=6280
  _NOTEBOOK._serialized_end=6562
  _JOB._serialized_start=6565
  _JOB._serialized_end=6816
  _DLTPIPELINE._serialized_start=6819
  _DLTPIPELINE._serialized_end=6975
  _CONSUMER._serialized_start=6978
  _CONSUMER._serialized_end=7149
  _FEATURETABLEFEATURES._serialized_start=7151
  _FEATURETABLEFEATURES._serialized_end=7219
  _TAG._serialized_start=7221
  _TAG._serialized_end=7254
  _MONITORINGCONFIG._serialized_start=7257
  _MONITORINGCONFIG._serialized_end=7401
  _PROFILE._serialized_start=7404
  _PROFILE._serialized_end=7616
  _TABLESTATS._serialized_start=7618
  _TABLESTATS._serialized_end=7648
  _SETMONITORINGCONFIG._serialized_start=7651
  _SETMONITORINGCONFIG._serialized_end=7907
  _SETMONITORINGCONFIG_RESPONSE._serialized_start=7800
  _SETMONITORINGCONFIG_RESPONSE._serialized_end=7869
  _GETMONITORINGCONFIG._serialized_start=7910
  _GETMONITORINGCONFIG._serialized_end=8069
  _GETMONITORINGCONFIG_RESPONSE._serialized_start=7800
  _GETMONITORINGCONFIG_RESPONSE._serialized_end=7869
  _LISTPROFILES._serialized_start=8072
  _LISTPROFILES._serialized_end=8276
  _LISTPROFILES_RESPONSE._serialized_start=8162
  _LISTPROFILES_RESPONSE._serialized_end=8238
  _GETPROFILE._serialized_start=8279
  _GETPROFILE._serialized_end=8430
  _GETPROFILE_RESPONSE._serialized_start=8342
  _GETPROFILE_RESPONSE._serialized_end=8392
  _CREATEPROFILE._serialized_start=8433
  _CREATEPROFILE._serialized_end=8693
  _CREATEPROFILE_RESPONSE._serialized_start=8342
  _CREATEPROFILE_RESPONSE._serialized_end=8392
  _DELETEPROFILES._serialized_start=8695
  _DELETEPROFILES._serialized_end=8813
  _DELETEPROFILES_RESPONSE._serialized_start=8736
  _DELETEPROFILES_RESPONSE._serialized_end=8775
  _FEATURESTORESERVICE._serialized_start=9176
  _FEATURESTORESERVICE._serialized_end=14587
FeatureStoreService = service_reflection.GeneratedServiceType('FeatureStoreService', (_service.Service,), dict(
  DESCRIPTOR = _FEATURESTORESERVICE,
  __module__ = 'feature_catalog_pb2'
  ))

FeatureStoreService_Stub = service_reflection.GeneratedServiceStubType('FeatureStoreService_Stub', (FeatureStoreService,), dict(
  DESCRIPTOR = _FEATURESTORESERVICE,
  __module__ = 'feature_catalog_pb2'
  ))


# @@protoc_insertion_point(module_scope)



from importlib.machinery import SourceFileLoader
import os
from setuptools import setup, find_namespace_packages

version = (
    SourceFileLoader(
        "databricks.feature_store.version",
        os.path.join("databricks", "feature_store", "version.py"),
    )
    .load_module()
    .VERSION
)

with open("README.md") as f:
    readme_contents = f.read()

with open("LICENSE.md") as f:
    license_contents = f.read()

# Append the text from LICENSE.md to the end of the README.md content
readme_contents_with_license = (
    readme_contents + "\n\n" + "# License" + "\n\n" + license_contents
)

try:
    # Overwrite the existing README.md file with the readme + license content for the setup() step.
    # The original version will be restored to it's original state after setup() finishes.
    with open("README.md", "w") as f:
        f.write(readme_contents_with_license)

    # TODO (ML-20365): DRY this with lookup client into universe/feature-store-commons
    def remove_comments_and_empty_lines(lines):
        def is_comment_or_empty(line):
            stripped = line.strip()
            return stripped == "" or stripped.startswith("#")

        return [line for line in lines if not is_comment_or_empty(line)]

    with open("requirements.txt", "r") as f:
        REQUIREMENTS = remove_comments_and_empty_lines(f.readlines())

    setup(
        name="databricks-feature-store",
        version=version,
        author_email="feedback@databricks.com",
        license="Databricks Proprietary License",
        license_files=("LICENSE.md", "NOTICE.md"),
        long_description=readme_contents_with_license,
        long_description_content_type="text/markdown",
        packages=find_namespace_packages(include=["databricks.*"], exclude=["tests"]),
        include_package_data=True,
        package_data={
            "databricks.feature_store.utils": ["headers.yaml"],
        },
        python_requires=">=3.7",
        install_requires=REQUIREMENTS,
        author="Databricks",
        description="Databricks Feature Store Client",
    )
finally:
    # Restore the README.md file to it's original state
    with open("README.md", "w") as f:
        f.write(readme_contents)



import json
import re
from typing import List, Optional

from py4j.protocol import Py4JJavaError
from pyspark.sql import DataFrame
from pyspark.sql.functions import lit, struct, udf
from pyspark.sql.types import StringType

from databricks.feature_store.utils.cosmosdb_type_utils import (
    get_data_type_converter,
    BASIC_DATA_TYPE_IDENTITIES,
)

PRIMARY_KEY_PROPERTY_NAME_VALUE = "_feature_store_internal__primary_keys"
FEATURE_STORE_SENTINEL_ID_VALUE = "_fs"

# Container provisioning configurations. Serverless accounts do not support offering a throughput.
# See the Cosmos DB Catalog API for use with creating databases and containers:
# https://github.com/Azure/azure-sdk-for-java/blob/main/sdk/cosmos/azure-cosmos-spark_3_2-12/docs/catalog-api.md
PROVISIONED_TABLE_PROPERTIES = f"TBLPROPERTIES(partitionKeyPath='/{PRIMARY_KEY_PROPERTY_NAME_VALUE}', indexingPolicy='OnlySystemProperties', autoScaleMaxThroughput='4000')"
SERVERLESS_TABLE_PROPERTIES = f"TBLPROPERTIES(partitionKeyPath='/{PRIMARY_KEY_PROPERTY_NAME_VALUE}', indexingPolicy='OnlySystemProperties')"


def generate_cosmosdb_primary_key(df: DataFrame, primary_keys: List[str]) -> DataFrame:
    """
    Generate the Feature Store internal primary key for Cosmos DB, and drop the individual primary keys.
    All DataFrame column data types are expected to be compatible with Cosmos DB.
    """
    return df.withColumn(
        PRIMARY_KEY_PROPERTY_NAME_VALUE,
        udf(json.dumps, StringType())(struct(*primary_keys)),
    ).drop(*primary_keys)


def generate_cosmosdb_id(
    df: DataFrame, column_for_id: Optional[str] = None
) -> DataFrame:
    """
    Generate the expected Cosmos DB id property. If provided, the `column_for_id` will be used as the `id` property.
    Otherwise, the Feature Store sentinel id value will be used.
    """
    id_col = (
        df[column_for_id] if column_for_id else lit(FEATURE_STORE_SENTINEL_ID_VALUE)
    )
    return df.withColumn("id", id_col)


def generate_cosmosdb_safe_data_types(df: DataFrame) -> DataFrame:
    """
    Helper function to convert a provided DataFrame for Cosmos DB publish.
    All unsupported data types, including nested ones, will converted to a Cosmos DB safe type.
    """
    # The column projections are either an existing column name or a converted column entity.
    columns = []
    for field in df.schema.fields:
        if type(field.dataType) in BASIC_DATA_TYPE_IDENTITIES:
            columns.append(field.name)
        else:
            converter = get_data_type_converter(field.dataType)
            to_cosmosdb_udf = udf(converter.to_cosmosdb, converter.online_data_type())
            columns.append(to_cosmosdb_udf(df[field.name]).alias(field.name))
    # Bulk projecting with df.select is faster than replacing columns inplace with df.withColumn.
    return df.select(*columns)


def validate_cosmosdb_name(name: str, name_type: str):
    """
    This is the same validation done in the backend. For Cosmos DB, we use Spark SQL to create the online store prior
    to any backend validation, so client-side validation is required to prevent SQL injections.
    """
    if not re.match(r"^\w+$", name):
        raise ValueError(
            f"The provided {name_type} must only contain alphanumeric or underscore characters."
        )


def create_database_statement(database_name: str) -> str:
    validate_cosmosdb_name(database_name, "database name")
    return f"CREATE DATABASE IF NOT EXISTS cosmosCatalog.{database_name}"


def create_table_statement(
    database_name: str, container_name: str, table_properties: str
) -> str:
    validate_cosmosdb_name(database_name, "database name")
    validate_cosmosdb_name(container_name, "container name")
    return f"CREATE TABLE IF NOT EXISTS cosmosCatalog.{database_name}.{container_name} using cosmos.oltp {table_properties}"


def is_serverless_throughput_offer_error(e: Exception) -> bool:
    """
    Determine if an exception was cause by offering a throughput to a serverless Cosmos DB account.
    TODO (ML-22040): Programmatically retrieve this info and refactor the Cosmos DB publish engine.
    """
    serverless_offer_error = "Setting offer throughput or autopilot on container is not supported for serverless accounts."
    if isinstance(e, Py4JJavaError) and serverless_offer_error in str(e.java_exception):
        return True
    return False



from databricks.feature_store.online_store_publish_client import (
    OnlineStorePublishClient,
    is_rdbms_spec,
    is_nosql_spec,
)
from databricks.feature_store.online_store_publish_rdbms_client import (
    OnlineStorePublishRdbmsClient,
)
from databricks.feature_store.online_store_publish_nosql_client import (
    OnlineStorePublishNoSqlClient,
)
from databricks.feature_store.online_store_spec import OnlineStoreSpec


def get_online_store_publish_client(
    online_store_spec: OnlineStoreSpec,
) -> OnlineStorePublishClient:
    if is_rdbms_spec(online_store_spec):
        return OnlineStorePublishRdbmsClient(online_store_spec)
    elif is_nosql_spec(online_store_spec):
        return OnlineStorePublishNoSqlClient(online_store_spec)
    raise ValueError(f"Unexpected online store type {type(online_store_spec)}")



from typing import List, Dict
from collections import defaultdict

from databricks.feature_store.entities.data_type import DataType
from databricks.feature_store.entities.feature_column_info import FeatureColumnInfo
from databricks.feature_store.entities.source_data_column_info import (
    SourceDataColumnInfo,
)
from databricks.feature_store.entities.feature_spec import FeatureSpec
from databricks.feature_store.entities.feature_table import FeatureTable

from databricks.feature_store.utils import utils

from pyspark.sql import DataFrame


def _spark_asof_join_features(
    df: DataFrame,
    df_lookup_keys: List[str],
    df_timestamp_lookup_key: str,
    feature_table_data: DataFrame,
    feature_table_keys: List[str],
    feature_table_timestamp_key: str,
    feature_to_output_name: Dict[str, str],
) -> DataFrame:
    # [ML-19825] Import TSDF only at as-of join time to avoid DynamoDB publish issue
    from tempo.tsdf import TSDF

    # Alias feature table's keys to DataFrame lookup keys
    ft_key_aliases = [
        feature_table_data[ft_key].alias(df_key)
        for (ft_key, df_key) in zip(feature_table_keys, df_lookup_keys)
    ]
    # Alias features to corresponding output names
    ft_features = [
        (feature_name, output_name)
        for feature_name, output_name in feature_to_output_name.items()
        # Skip join if feature it is already in DataFrame and therefore overridden
        if output_name not in df.columns
    ]
    ft_feature_aliases = [
        feature_table_data[feature_name].alias(output_name)
        for feature_name, output_name in ft_features
    ]
    # Alias feature table's timestamp key to DataFrame timestamp lookup keys
    ft_timestamp_key_aliases = [
        feature_table_data[feature_table_timestamp_key].alias(df_timestamp_lookup_key)
    ]
    # Select key, timestamp key, and feature columns from feature table
    feature_and_keys = feature_table_data.select(
        ft_key_aliases + ft_timestamp_key_aliases + ft_feature_aliases
    )
    # Initialize Tempo TSDFs for df and feature table.
    # Feature table timestamp keys and primary keys are aliased to
    # DataFrame's timestamp lookup keys and lookup keys respectively
    # because ts_col and partition_cols must match.
    df_tsdf = TSDF(df, ts_col=df_timestamp_lookup_key, partition_cols=df_lookup_keys)
    ft_tsdf = TSDF(
        feature_and_keys,
        ts_col=df_timestamp_lookup_key,
        partition_cols=df_lookup_keys,
    )
    # Perform as-of join
    joined_df = df_tsdf.asofJoin(
        ft_tsdf, left_prefix="left", right_prefix="right", skipNulls=False
    ).df
    # Remove prefixes from joined df to restore column names
    left_aliases = [
        joined_df[f"left_{column_name}"].alias(column_name)
        for column_name in df.columns
        if column_name not in df_lookup_keys
    ]
    right_aliases = [
        joined_df[f"right_{output_name}"].alias(output_name)
        for (_, output_name) in ft_features
    ]
    return joined_df.select(df_lookup_keys + left_aliases + right_aliases)


def _spark_join_features(
    df: DataFrame,
    df_keys: List[str],
    feature_table_data: DataFrame,
    feature_table_keys: List[str],
    feature_to_output_name: Dict[str, str],
) -> DataFrame:
    """
    Helper to join `feature_name` from `feature_table_data` into `df`.

    This join uses a temporary table that contains only the keys and feature
    from the feature table. The temporary table aliases the keys to match
    the lookup keys and the feature to match the output_name.

    Aliasing the keys allows us to join on name instead of by column,
    which prevents duplicate column names after the join.
    (see: https://kb.databricks.com/data/join-two-dataframes-duplicated-columns.html)

    The joined-in feature is guaranteed to be unique because FeatureSpec
    columns must be unique and the join is skipped if the feature
    already exists in the DataFrame.
    """

    # Alias feature table's keys to DataFrame lookup keys
    ft_key_aliases = [
        feature_table_data[ft_key].alias(df_key)
        for (ft_key, df_key) in zip(feature_table_keys, df_keys)
    ]
    # Alias features to corresponding output names
    ft_feature_aliases = [
        feature_table_data[feature_name].alias(output_name)
        for feature_name, output_name in feature_to_output_name.items()
        # Skip join if feature it is already in DataFrame and therefore overridden
        if output_name not in df.columns
    ]
    # Select key and feature columns from feature table
    feature_and_keys = feature_table_data.select(ft_key_aliases + ft_feature_aliases)
    # Join feature to feature table
    return df.join(feature_and_keys, df_keys, how="left")


def _validate_join_keys(
    feature_column_info: FeatureColumnInfo,
    df: DataFrame,
    feature_table_metadata: FeatureTable,
    feature_table_data: DataFrame,
    is_timestamp_key: bool = False,
):
    join_error_phrase = (
        f"Unable to join feature table '{feature_column_info.table_name}'"
    )
    feature_column_info_keys = (
        feature_column_info.timestamp_lookup_key
        if is_timestamp_key
        else feature_column_info.lookup_key
    )
    feature_table_keys = (
        feature_table_metadata.timestamp_keys
        if is_timestamp_key
        else feature_table_metadata.primary_keys
    )

    lookup_key_kind = "timestamp lookup key" if is_timestamp_key else "lookup key"
    feature_table_key_kind = "timestamp key" if is_timestamp_key else "primary key"

    # Validate df has necessary keys
    missing_df_keys = list(
        filter(lambda df_key: df_key not in df.columns, feature_column_info_keys)
    )
    if missing_df_keys:
        missing_keys = ", ".join([f"'{key}'" for key in missing_df_keys])
        raise ValueError(
            f"{join_error_phrase} because {lookup_key_kind} {missing_keys} not found in DataFrame."
        )
    # Validate feature table has necessary keys
    missing_ft_keys = list(
        filter(
            lambda ft_key: ft_key not in feature_table_data.columns, feature_table_keys
        )
    )
    if missing_ft_keys:
        missing_keys = ", ".join([f"'{key}'" for key in missing_ft_keys])
        raise ValueError(
            f"{join_error_phrase} because {feature_table_key_kind} {missing_keys} not found in feature table."
        )

    # Validate number of feature table keys matches number of df lookup keys
    if len(feature_column_info_keys) != len(feature_table_keys):
        raise ValueError(
            f"{join_error_phrase} because "
            f"number of {feature_table_key_kind}s ({feature_table_keys}) "
            f"does not match "
            f"number of {lookup_key_kind}s ({feature_column_info_keys})."
        )

    # Validate feature table keys match types of df keys. The number of keys is expected to be the same.
    for (df_key, ft_key) in zip(feature_column_info_keys, feature_table_keys):
        df_key_type = DataType.from_spark_type(df.schema[df_key].dataType)
        ft_key_type = DataType.from_spark_type(
            feature_table_data.schema[ft_key].dataType
        )
        if df_key_type != ft_key_type:
            raise ValueError(
                f"{join_error_phrase} because {feature_table_key_kind} '{ft_key}' has type '{DataType.to_string(ft_key_type)}' "
                f"but corresponding {lookup_key_kind} '{df_key}' has type '{DataType.to_string(df_key_type)}' in DataFrame."
            )


def _validate_join_feature_data(
    feature_spec: FeatureSpec,
    df: DataFrame,
    extra_columns: List[str],
    feature_table_metadata_map: Dict[str, FeatureTable],
    feature_table_data_map: Dict[str, DataFrame],
):
    # Validate extra columns are unique
    if len(extra_columns) != len(set(extra_columns)):
        raise ValueError("extra_columns must be unique")
    # Validate df has extra columns
    for extra_column in extra_columns:
        if extra_column not in df.columns:
            raise ValueError(f"Column '{extra_column}' not found in DataFrame")
    for feature_info in feature_spec.feature_column_infos:
        feature_table_metadata = feature_table_metadata_map[feature_info.table_name]
        feature_table_data = feature_table_data_map[feature_info.table_name]
        # Validate feature table primary keys match length/type of df lookup keys
        _validate_join_keys(
            feature_info,
            df,
            feature_table_metadata,
            feature_table_data,
            is_timestamp_key=False,
        )
        # Validate feature table timestamp keys match length/type of df timestamp lookup keys
        _validate_join_keys(
            feature_info,
            df,
            feature_table_metadata,
            feature_table_data,
            is_timestamp_key=True,
        )


def join_feature_data(
    feature_spec: FeatureSpec,
    df: DataFrame,
    extra_columns: List[str],
    feature_table_metadata_map: Dict[str, FeatureTable],
    feature_table_data_map: Dict[str, DataFrame],
) -> DataFrame:
    """
    Joins `df` with features specified by `feature_spec`,
    using `feature_table_metadata_map` to determine join key
    and `feature_table_data_map` to retrieve the feature data.

    Before joining, it checks that
    1) `extra_columns` are in `df`
    2) `extra_columns` are unique
    3) feature table keys match length and types of `df` lookup keys specified by FeatureSpec
    4) `df` contains lookup keys specified by FeatureSpec
    5) feature table timestamp lookup keys match length and types of `df` timestamp lookup keys if specified by FeatureSpec
    6) `df` contains timestamp lookup keys if specified by FeatureSpec
    """
    _validate_join_feature_data(
        feature_spec=feature_spec,
        df=df,
        extra_columns=extra_columns,
        feature_table_metadata_map=feature_table_metadata_map,
        feature_table_data_map=feature_table_data_map,
    )

    # Helper class to group all unique combinations of feature table names and lookup keys.
    # All features in each of these groups will be JOINed with the training df using a single JOIN.
    class JoinDataKey:
        def __init__(
            self,
            feature_table: str,
            lookup_key: List[str],
            timestamp_lookup_key: List[str],
        ):
            self.feature_table = feature_table
            self.lookup_key = lookup_key
            self.timestamp_lookup_key = timestamp_lookup_key

        def __hash__(self):
            return (
                hash(self.feature_table)
                + hash(tuple(self.lookup_key))
                + hash(tuple(self.timestamp_lookup_key))
            )

        def __eq__(self, other):
            return (
                self.feature_table == other.feature_table
                and self.lookup_key == other.lookup_key
                and self.timestamp_lookup_key == other.timestamp_lookup_key
            )

    # Iterate through the list of FeatureColumnInfo and group features by name of the
    # feature table and lookup key(s) and timestamp lookup key(s)
    table_join_data = defaultdict(dict)
    for feature_info in feature_spec.feature_column_infos:
        join_data_key = JoinDataKey(
            feature_info.table_name,
            feature_info.lookup_key,
            feature_info.timestamp_lookup_key,
        )
        table_join_data[join_data_key][
            feature_info.feature_name
        ] = feature_info.output_name

    for join_data_key, feature_to_output_name in table_join_data.items():

        feature_table_metadata = feature_table_metadata_map[join_data_key.feature_table]
        feature_table_data = feature_table_data_map[join_data_key.feature_table]

        if join_data_key.timestamp_lookup_key:
            df = _spark_asof_join_features(
                df=df,
                df_lookup_keys=join_data_key.lookup_key,
                df_timestamp_lookup_key=join_data_key.timestamp_lookup_key[0],
                feature_table_data=feature_table_data,
                feature_table_keys=feature_table_metadata.primary_keys,
                feature_table_timestamp_key=feature_table_metadata.timestamp_keys[0],
                feature_to_output_name=feature_to_output_name,
            )
        else:
            df = _spark_join_features(
                df=df,
                df_keys=join_data_key.lookup_key,
                feature_table_data=feature_table_data,
                feature_table_keys=feature_table_metadata.primary_keys,
                feature_to_output_name=feature_to_output_name,
            )
    # Return the requested columns in the feature spec order with extra_columns appended at the end
    feature_spec_column_names = [info.output_name for info in feature_spec.column_infos]
    extra_column_names = [
        col for col in extra_columns if col not in feature_spec_column_names
    ]

    return df.select(feature_spec_column_names + extra_column_names)



from collections import defaultdict
from tempfile import TemporaryDirectory
from typing import List, Dict
import itertools
import os

from mlflow import sagemaker as mlflow_sagemaker
from mlflow.utils import databricks_utils
from mlflow.pyfunc import DATA

from databricks.feature_store.catalog_client import CatalogClient
from databricks.feature_store.constants import MODEL_DATA_PATH_ROOT
from databricks.feature_store.entities.feature_spec import FeatureSpec
from databricks.feature_store.entities.feature_tables_for_serving import (
    FeatureTablesForSageMakerServing,
)
from databricks.feature_store.entities.online_feature_table import (
    OnlineFeatureTable,
    OnlineFeatureTableForSageMakerServing,
)
from databricks.feature_store.entities.online_store_for_serving import (
    OnlineStoreForSageMakerServing,
)

from databricks.feature_store.entities.store_type import StoreType
from databricks.feature_store.utils import request_context
from databricks.feature_store.utils.request_context import RequestContext
from databricks.feature_store.utils.utils import download_model_artifacts


def _convert_olft_to_sagemaker_olft(
    olft: OnlineFeatureTable,
) -> OnlineFeatureTableForSageMakerServing:
    """
    Transforms an OnlineFeatureTable to an OnlineFeatureTableForSageMakerServing

    :param olft: The source OnlineFeatureTable. Must have store_type DYNAMODB.
    """
    online_store = olft.online_store
    if not online_store.store_type == StoreType.DYNAMODB:
        raise Exception(
            "Internal Error: Attempted to convert online feature table with store type "
            f" {online_store.store_type} to OnlineFeatureTableForSageMakerServing."
        )
    online_store_for_sagemaker_serving = OnlineStoreForSageMakerServing(
        creation_timestamp_ms=online_store.creation_timestamp_ms,
        # Only DynamoDB online stores are expected at this point, so extra_configs
        # is a DynamoDbConf.
        extra_configs=online_store.extra_configs,
        query_mode=online_store.query_mode,
    )
    return OnlineFeatureTableForSageMakerServing(
        feature_table_name=olft.feature_table_name,
        online_feature_table_name=olft.online_feature_table_name,
        online_store=online_store_for_sagemaker_serving,
        primary_keys=olft.primary_keys,
        feature_table_id=olft.feature_table_id,
        features=olft.features,
        timestamp_keys=olft.timestamp_keys,
    )


def _generate_sagemaker_model_serving_metadata(
    feature_table_to_features: Dict[str, List[str]],
    online_feature_tables: List[OnlineFeatureTable],
) -> FeatureTablesForSageMakerServing:
    """
    Generates online feature table metadata required by SageMaker serving.

    :param feature_table_to_features: Dictionary of required features per feature table. eg
      {"prod.user_features": ["age", "sex", "bmi", ...]
    :param online_feature_tables: Online feature tables available, returned by
      GetModelServingMetdata
    :return: FeatureTablesForSageMakerServing object
    """
    # Group candidate online feature tables by feature table name
    feature_table_to_online_feature_tables = {
        k: list(v)
        for k, v in itertools.groupby(
            online_feature_tables, lambda olft: olft.feature_table_name
        )
    }
    # For each required feature table, select up to one online feature table
    selected_online_feature_table_or_none = {}
    for ft_name in feature_table_to_features.keys():
        all_olfts = feature_table_to_online_feature_tables.get(ft_name, [])
        supported_olfts = [
            olft
            for olft in all_olfts
            if olft.online_store.store_type == StoreType.DYNAMODB
        ]
        # If there are multiple options, choose the oldest
        ts_sorted_olfts = sorted(
            supported_olfts, key=lambda olft: olft.online_store.creation_timestamp_ms
        )
        selected_online_feature_table_or_none[ft_name] = (
            ts_sorted_olfts[0] if ts_sorted_olfts else None
        )

    # Throw if there is no online feature table
    missing_online_feature_tables = [
        ft
        for (ft, olft) in selected_online_feature_table_or_none.items()
        if olft is None
    ]
    if missing_online_feature_tables:
        missing_table_strs = [
            f"\n\tFeature Table: {ft_name}\n\t\tRequired Features:{feature_table_to_features[ft_name]}"
            for ft_name in missing_online_feature_tables
        ]
        missing_tables_str = "\n\n".join(missing_table_strs)
        feature_table_str = (
            "feature_table"
            if len(missing_online_feature_tables) <= 1
            else "feature_tables"
        )
        raise ValueError(
            f"No valid online stores were found for {feature_table_str} {missing_online_feature_tables}. "
            "In order to deploy a model to SageMaker, feature tables must be published to a DynamoDB online store, "
            "including all required features. The following feature tables were not published to DynamoDB or are missing"
            " features in the online store:"
            f"{missing_tables_str}"
        )

    online_feature_tables_for_sagemaker_serving = [
        _convert_olft_to_sagemaker_olft(olft)
        for olft in selected_online_feature_table_or_none.values()
    ]

    return FeatureTablesForSageMakerServing(
        online_feature_tables=online_feature_tables_for_sagemaker_serving
    )


def _add_model_serving_artifact(model_data_path: str):
    """
    Retrieve and save model serving metadata from the Feature Catalog.

    :param model_data_path: Path to the model's data directory. eg
      "/tmp/the_model/data/feature_store"
    """
    feature_spec = FeatureSpec.load(model_data_path)
    feature_table_to_features = defaultdict(list)
    for fci in feature_spec.feature_column_infos:
        feature_table_to_features[fci.table_name].append(fci.feature_name)

    if feature_table_to_features:
        catalog_client = CatalogClient(databricks_utils.get_databricks_host_creds)
        online_feature_tables = catalog_client.get_model_serving_metadata(
            feature_table_to_features,
            req_context=RequestContext(request_context.GET_MODEL_SERVING_METADATA),
        )
    else:
        online_feature_tables = []

    sagemaker_model_serving_metadata = _generate_sagemaker_model_serving_metadata(
        feature_table_to_features, online_feature_tables
    )
    s = sagemaker_model_serving_metadata.to_proto().SerializeToString()

    with open(
        os.path.join(model_data_path, FeatureTablesForSageMakerServing.DATA_FILE), "wb"
    ) as f:
        f.write(s)


def deploy(app_name: str, model_uri: str, execution_role_arn: str = None, **kwargs):
    """
    Deploy an MLflow model on AWS SageMaker. The IAM role that your Databricks cluster is running
    under must have sufficient permissions to deploy a SageMaker model.

    The model must have been logged with :meth:`FeatureStoreClient.log_model`.

    :param app_name: Name of the deployed application.
    :param model_uri: The location, in URI format, of the MLflow model logged using
          :meth:`FeatureStoreClient.log_model`. One of:

            * ``runs:/<mlflow_run_id>/run-relative/path/to/model``

            * ``models:/<model_name>/<model_version>``

            * ``models:/<model_name>/<stage>``

    :param execution_role_arn: The name of an IAM role granting the SageMaker service permissions to
                               access the specified Docker image and S3 bucket containing MLflow
                               model artifacts. If unspecified, the currently-assumed role will be
                               used. This execution role is passed to the SageMaker service when
                               creating a SageMaker model from the specified MLflow model. It is
                               passed as the ``ExecutionRoleArn`` parameter of the `SageMaker
                               CreateModel API call <https://docs.aws.amazon.com/sagemaker/latest/
                               dg/API_CreateModel.html>`_. This role is *not* assumed for any other
                               call. For more information about SageMaker execution roles for model
                               creation, see
                               https://docs.aws.amazon.com/sagemaker/latest/dg/sagemaker-roles.html.

    Additional keyword arguments will be passed through to :func:`mlflow.sagemaker.deploy`.
    """
    with TemporaryDirectory() as tmp_dir:
        download_model_artifacts(model_uri, tmp_dir)
        # Augment model model artifacts with an additional model serving artifact. This will be
        # used by SageMaker to identify and connect to online stores for feature lookup.
        _add_model_serving_artifact(
            model_data_path=os.path.join(tmp_dir, DATA, MODEL_DATA_PATH_ROOT)
        )
        mlflow_sagemaker.deploy(
            app_name=app_name,
            model_uri=tmp_dir,
            execution_role_arn=execution_role_arn,
            **kwargs,
        )



import logging
from databricks.feature_store.utils.sql import is_qualified_table_name
from pyspark.sql import DataFrame

_logger = logging.getLogger(__name__)


def standardize_checkpoint_location(checkpoint_location):
    if checkpoint_location is None:
        return checkpoint_location
    checkpoint_location = checkpoint_location.strip()
    if checkpoint_location == "":
        checkpoint_location = None
    return checkpoint_location


def check_qualified_name(feature_table_name):
    if not is_qualified_table_name(feature_table_name):
        raise ValueError(
            "Feature table name must have the form <database_name>.<table_name> "
            "and only contain alphabet characters, numbers and _."
        )


def check_dataframe_type(df):
    """
    Check if df is a PySpark DataFrame, otherwise raise an error.
    """
    if not isinstance(df, DataFrame):
        raise ValueError(
            f"Unsupported DataFrame type: {type(df)}. DataFrame must be a PySpark DataFrame."
        )


def check_kwargs_empty(the_kwargs, method_name):
    if len(the_kwargs) != 0:
        raise TypeError(
            f"{method_name}() got unexpected keyword argument(s): {list(the_kwargs.keys())}"
        )



from databricks.feature_store.entities._feature_store_object import _FeatureStoreObject
from databricks.feature_store.entities.data_type import DataType


class Feature(_FeatureStoreObject):
    def __init__(
        self, feature_table, name, data_type, description, data_type_details=None
    ):
        self._feature_table = feature_table
        self._name = name
        self._data_type = data_type
        self._description = description
        self._data_type_details = data_type_details

    @property
    def feature_table(self):
        return self._feature_table

    @property
    def name(self):
        return self._name

    @property
    def data_type(self):
        return self._data_type

    @property
    def data_type_details(self):
        return self._data_type_details

    @property
    def description(self):
        return self._description

    @classmethod
    def from_proto(cls, feature_proto):
        return cls(
            feature_table=feature_proto.table,
            name=feature_proto.name,
            data_type=DataType.to_string(feature_proto.data_type),
            data_type_details=feature_proto.data_type_details,
            description=feature_proto.description,
        )



import abc
from datetime import timedelta
from typing import Optional

from databricks.feature_store.entities._feature_store_object import _FeatureStoreObject
from databricks.feature_store.entities.store_type import StoreType
from databricks.feature_store.online_store_spec import OnlineStoreSpec
from databricks.feature_store.online_store_spec.online_store_properties import (
    DATABASE_NAME,
    TABLE_NAME,
    CONTAINER_NAME,
)
from databricks.feature_store.protos.feature_store_serving_pb2 import (
    MySqlMetadata as ProtoMySqlMetadata,
    SqlServerMetadata as ProtoSqlServerMetadata,
    DynamoDbMetadata as ProtoDynamoDbMetadata,
    CosmosDbMetadata as ProtoCosmosDbMetadata,
)
from databricks.feature_store.online_store_publish_client import is_rdbms_spec


def get_online_store_name(online_store: OnlineStoreSpec):
    """
    Derives and returns the online store name based on the provided online store type.
    Assumes that the online store spec is well formed (e.g. augmented with the relevant database/table names)
    """
    if is_rdbms_spec(online_store):
        return f"{online_store.database_name}.{online_store.table_name}"
    elif online_store.store_type == StoreType.DYNAMODB:
        return online_store.table_name
    elif online_store.store_type == StoreType.COSMOSDB:
        return f"{online_store.database_name}.{online_store.container_name}"
    else:
        raise ValueError(f"Unexpected online store type {type(online_store)}")


def augment_online_store(online_store: OnlineStoreSpec, feature_table_name: str):
    """
    Apply default values for member variables of online_store that are None. This currently includes:
      - RDBMS stores: database name, table name = hive_database_name, hive_table_name
      - DynamoDB: table name = hive_database_name.hive_table_name
      - Cosmos DB: database name, container name = hive_database_name, hive_table_name
    Throws if the provided online_store is invalid.
    """
    hive_database_name, hive_table_name = feature_table_name.split(".")

    updated_metadata = {}
    if is_rdbms_spec(online_store) or online_store.store_type == StoreType.COSMOSDB:
        # SQL stores use table while Cosmos DB uses container terminology.
        TABLE_OR_CONTAINER, table_or_container_name = (
            (TABLE_NAME, online_store.table_name)
            if is_rdbms_spec(online_store)
            else (CONTAINER_NAME, online_store.container_name)
        )
        if (online_store.database_name is None) != (table_or_container_name is None):
            raise ValueError(
                f"The OnlineStoreSpec {online_store.store_type} must specify either both database_name "
                f"and {TABLE_OR_CONTAINER}, or neither."
            )
        elif online_store.database_name is None and table_or_container_name is None:
            updated_metadata[DATABASE_NAME] = hive_database_name
            updated_metadata[TABLE_OR_CONTAINER] = hive_table_name
    elif online_store.store_type == StoreType.DYNAMODB:
        if online_store.table_name is None:
            updated_metadata[TABLE_NAME] = f"{hive_database_name}.{hive_table_name}"
    else:
        raise ValueError(f"Unexpected online store type {type(online_store)}")
    return online_store.clone(**updated_metadata)


class OnlineStoreAdditionalMetadata(_FeatureStoreObject):
    """
    Abstract class for defining online store additional metadata.
    """

    pass


class MySqlMetadata(OnlineStoreAdditionalMetadata):
    def __init__(self, host: str, port: int):
        self._host = host
        self._port = port

    @property
    def host(self):
        return self._host

    @property
    def port(self):
        return self._port

    def to_proto(self):
        return ProtoMySqlMetadata(
            host=self.host,
            port=self.port,
        )

    @classmethod
    def from_proto(cls, mysql_metadata_proto: ProtoMySqlMetadata):
        return MySqlMetadata(mysql_metadata_proto.host, mysql_metadata_proto.port)


class SqlServerMetadata(OnlineStoreAdditionalMetadata):
    def __init__(self, host: str, port: int):
        self._host = host
        self._port = port

    @property
    def host(self):
        return self._host

    @property
    def port(self):
        return self._port

    def to_proto(self):
        return ProtoSqlServerMetadata(
            host=self.host,
            port=self.port,
        )

    @classmethod
    def from_proto(cls, sql_server_metadata_proto: ProtoSqlServerMetadata):
        return SqlServerMetadata(
            sql_server_metadata_proto.host, sql_server_metadata_proto.port
        )


class DynamoDbMetadata(OnlineStoreAdditionalMetadata):
    def __init__(self, region: str, table_arn: str, ttl: Optional[timedelta]):
        self._region = region
        self._table_arn = table_arn
        self._ttl = ttl

    @property
    def region(self):
        return self._region

    @property
    def table_arn(self):
        return self._table_arn

    @property
    def ttl(self) -> Optional[timedelta]:
        return self._ttl

    def to_proto(self):
        proto_ttl = int(self.ttl.total_seconds()) if self.ttl else None
        return ProtoDynamoDbMetadata(
            region=self.region, table_arn=self.table_arn, ttl=proto_ttl
        )

    @classmethod
    def from_proto(cls, dynamodb_metadata_proto: ProtoDynamoDbMetadata):
        ttl = (
            timedelta(seconds=dynamodb_metadata_proto.ttl)
            if dynamodb_metadata_proto.HasField("ttl")
            else None
        )
        return DynamoDbMetadata(
            dynamodb_metadata_proto.region, dynamodb_metadata_proto.table_arn, ttl
        )


class CosmosDbMetadata(OnlineStoreAdditionalMetadata):
    def __init__(self, account_uri: str, container_uri: str):
        self._account_uri = account_uri
        self._container_uri = container_uri

    @property
    def account_uri(self):
        return self._account_uri

    @property
    def container_uri(self):
        return self._container_uri

    def to_proto(self):
        return ProtoCosmosDbMetadata(
            account_uri=self.account_uri, container_uri=self.container_uri
        )

    @classmethod
    def from_proto(cls, cosmosdb_metadata_proto):
        # The current MLR does not support Cosmos DB TTL. To ensure backwards incompatibility when future MLRs
        # introduce TTL support, throw here if an online store with a defined TTL is retrieved.
        if cosmosdb_metadata_proto.HasField("ttl"):
            raise ValueError(
                "Cosmos DB time-to-live (TTL) is not supported by this version of the databricks-feature-store library."
            )
        return CosmosDbMetadata(
            account_uri=cosmosdb_metadata_proto.account_uri,
            container_uri=cosmosdb_metadata_proto.container_uri,
        )


class OnlineStoreMetadata(abc.ABC):
    def __init__(self, online_store: OnlineStoreSpec, cloud_provider_unique_id: str):
        """
        Class representing the online store metadata that will be published to the Feature Catalog.
        Assumes that the online store spec is well formed (e.g. augmented with the relevant database/table names)
        """
        self._cloud = online_store.cloud
        self._store_type = online_store.store_type
        self._read_secret_prefix = online_store.read_secret_prefix
        self._write_secret_prefix = online_store.write_secret_prefix
        if (
            self._store_type == StoreType.AURORA_MYSQL
            or self._store_type == StoreType.MYSQL
        ):
            self._additional_metadata = MySqlMetadata(
                online_store.hostname, online_store.port
            )
        elif self._store_type == StoreType.SQL_SERVER:
            self._additional_metadata = SqlServerMetadata(
                online_store.hostname, online_store.port
            )
        elif self._store_type == StoreType.DYNAMODB:
            self._additional_metadata = DynamoDbMetadata(
                online_store.region, cloud_provider_unique_id, online_store.ttl
            )
        elif self._store_type == StoreType.COSMOSDB:
            self._additional_metadata = CosmosDbMetadata(
                account_uri=online_store.account_uri,
                container_uri=cloud_provider_unique_id,
            )
        self._online_table = get_online_store_name(online_store)

    @property
    def online_table(self):
        return self._online_table

    @property
    def cloud(self):
        return self._cloud

    @property
    def store_type(self):
        return self._store_type

    @property
    def read_secret_prefix(self):
        return self._read_secret_prefix

    @property
    def write_secret_prefix(self):
        return self._write_secret_prefix

    @property
    def additional_metadata(self) -> OnlineStoreAdditionalMetadata:
        return self._additional_metadata
